﻿using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Xml;

namespace com.swivelsecure.pinsafe {
    public class TextSettings {
        // Names for strings:
        private const string USERNAME = "username";
        private const string PASSWORD = "password";
        private const string OTC = "otc";
        private const string SHOW_IMAGE = "show_image";
        private const string LOGIN = "login";
        private const string LOGO = "logo";
        private const string PAGE_TITLE = "page_title";
        private const string PAGE_HEADER = "page_header";
        private const string PAGE_FOOTER = "page_footer";
        private const string PINSAFE_FAILED = "pinsafe_failed";
        private const string SHAREPOINT_FAILED = "sharepoint_failed";
        private const string OLD_OTC = "old_otc";
        private const string NEW_OTC1 = "new_otc_1";
        private const string NEW_OTC2 = "new_otc_2";
        private const string CHANGE_PIN = "change_pin";
        private const string CHANGE_PIN_PAGE_TITLE = "change_pin_page_title";
        private const string CHANGE_PIN_PAGE_HEADER = "change_pin_page_header";
        private const string CHANGE_PIN_PAGE_FOOTER = "change_pin_page_footer";
        private const string CHANGE_PIN_FAILED = "change_pin_failed";
        private const string OLD_PASSWORD = "old_password";
        private const string NEW_PASSWORD1 = "new_password_1";
        private const string NEW_PASSWORD2 = "new_password_2";
        private const string CHANGE_PASSWORD = "change_password";
        private const string CHANGE_PASSWORD_PAGE_TITLE = "change_password_page_title";
        private const string CHANGE_PASSWORD_PAGE_HEADER = "change_password_page_header";
        private const string CHANGE_PASSWORD_PAGE_FOOTER = "change_password_page_footer";
        private const string CHANGE_PASSWORD_FAILED = "change_password_failed";
		private const string CHANGE_PASSWORD_DISABLED = "change_password_disabled";
		private const string USER_UNKNOWN = "user_unknown";
        private const string INCORRECT_PASSWORD = "incorrect_password";
        private const string PASSWORD_POLICY_ERROR = "password_policy_error";
        private const string USERNAME_MISSING = "username_missing";
        private const string PASSWORD_MISSING = "password_missing";
        private const string OTC_MISSING = "otc_missing";
	    private const string RESET_CODE = "reset_code";
	    private const string RESET_CODE_BUTTON = "reset_code_button";
	    private const string RESET_BUTTON = "reset_button";
	    private const string RESET_PAGE_TITLE = "reset_page_title";
	    private const string RESET_PAGE_HEADER = "reset_page_header";
	    private const string RESET_PAGE_FOOTER = "reset_page_footer";
	    private const string RESET_PIN_SUCCESS = "reset_pin_success";
		private const string RESET_PASSWORD_SUCCESS = "reset_password_success";
		private const string RESET_PIN_FAILED = "reset_pin_failed";
		private const string RESET_PASSWORD_FAILED = "reset_password_failed";
		private const string RESET_PWD_ALERT = "reset_pin_alert";
		private const string RESET_DISABLED = "reset_disabled";
	    private const string FORGOTTEN_PIN = "reset_pin_request";
		private const string FORGOTTEN_PASSWORD = "reset_pwd_request";
		private const string FORGOTTEN_PIN_PASSWORD = "reset_pin_pwd_request";

        // Default values
        private const string DEFAULT_USERNAME = "Username";
        private const string DEFAULT_PASSWORD = "Password";
        private const string DEFAULT_OTC = "OTC";
        private const string DEFAULT_SHOW_IMAGE = "Show Image";
        private const string DEFAULT_LOGIN = "Log In";
        private const string DEFAULT_LOGO = "./logo.gif";
        private const string DEFAULT_PAGE_TITLE = "Login Page";
        private const string DEFAULT_PAGE_HEADER = "Please Log In";
        private const string DEFAULT_PAGE_FOOTER = "";
        private const string DEFAULT_PINSAFE_FAILED = "PINsafe authentication failed";
        private const string DEFAULT_SHAREPOINT_FAILED = "Sharepoint authentication failed";
        private const string DEFAULT_OLD_OTC = "Old OTC";
        private const string DEFAULT_NEW_OTC1 = "New OTC";
        private const string DEFAULT_NEW_OTC2 = "Confirm OTC";
        private const string DEFAULT_CHANGE_PIN = "Change";
        private const string DEFAULT_CHANGE_PIN_PAGE_TITLE = "Change PIN";
        private const string DEFAULT_CHANGE_PIN_PAGE_HEADER = "Change PIN";
        private const string DEFAULT_CHANGE_PIN_PAGE_FOOTER = "";
        private const string DEFAULT_CHANGE_PIN_FAILED = "PIN Change failed";
        private const string DEFAULT_OLD_PASSWORD = "Old Password";
        private const string DEFAULT_NEW_PASSWORD1 = "New Password";
        private const string DEFAULT_NEW_PASSWORD2 = "Confirm Password";
        private const string DEFAULT_CHANGE_PASSWORD = "Change";
        private const string DEFAULT_CHANGE_PASSWORD_PAGE_TITLE = "Change Password";
        private const string DEFAULT_CHANGE_PASSWORD_PAGE_HEADER = "Change Password";
        private const string DEFAULT_CHANGE_PASSWORD_PAGE_FOOTER = "";
        private const string DEFAULT_CHANGE_PASSWORD_FAILED = "Password Change failed";
		private const string DEFAULT_CHANGE_PASSWORD_DISABLED = "Password Change is disabled";
		private const string DEFAULT_USER_UNKNOWN = "User \"{0}\" not found";
        private const string DEFAULT_INCORRECT_PASSWORD = "The old password is incorrect";
        private const string DEFAULT_PASSWORD_POLICY_ERROR = 
            "The new password does not meet the password policy. Possible reasons are:<br/>" +
            "Too short<br/>" +
            "Not complex enough<br/>" +
            "Re-use of previous password<br/>" +
            "Change too soon after previous change";
        private const string DEFAULT_USERNAME_MISSING = "No username has been entered";
        private const string DEFAULT_PASSWORD_MISSING = "No password has been entered";
        private const string DEFAULT_OTC_MISSING = "No one-time code has been entered";
		private const string DEFAULT_RESET_CODE = "Reset Code";
		private const string DEFAULT_RESET_CODE_BUTTON = "Get Reset Code";
		private const string DEFAULT_RESET_BUTTON = "Reset";
		private const string DEFAULT_RESET_PAGE_TITLE = "PIN / Password Reset";
		private const string DEFAULT_RESET_PAGE_HEADER = "PIN / Password Reset";
		private const string DEFAULT_RESET_PAGE_FOOTER = "";
		private const string DEFAULT_RESET_PIN_SUCCESS = "PIN reset for user \"{0}\"";
		private const string DEFAULT_RESET_PASSWORD_SUCCESS = "Password reset for user \"{0}\"";
		private const string DEFAULT_RESET_PIN_FAILED = "PIN reset failed for user \"{0}\": {1}";
		private const string DEFAULT_RESET_PASSWORD_FAILED = "Password reset failed for user \"{0}\": {1}";
	    private const string DEFAULT_RESET_PWD_ALERT = "Your password has been reset to \"{0}\"";
	    private const string DEFAULT_RESET_DISABLED = "PIN/Password reset is not enabled";
		private const string DEFAULT_FORGOTTEN_PIN = "Forgotten PIN";
		private const string DEFAULT_FORGOTTEN_PASSWORD = "Forgotten password";
		private const string DEFAULT_FORGOTTEN_PIN_PASSWORD = "Forgotten PIN / password";

        private static readonly string[] CUSTOM_ERROR_CODES = new[] {
                                                                     "__CHANGE_PIN_NO_SECURITY_STRING",
                                                                     "__AGENT_ERROR_USER_LOCKED",
                                                                     "__CHANGE_PIN_PIN_ERROR",
                                                                     "__AGENT_ERROR_NO_CHANGE",
                                                                     "__AGENT_ERROR_PIN_SIZE",
                                                                     "__AGENT_ERROR_PIN_COMPOSITION",
                                                                     "__AGENT_WARN_CHANGE_PIN",
                                                                     "__AGENT_ERROR_PIN_EXPIRED"
        };
        private static readonly string[] CUSTOM_ERROR_MESSAGES = new[] {
                                                                        "Your account has been locked. Please contact the system administrator.",
                                                                        "Your account has been locked. Please contact the system administrator.",
                                                                        "The Old OTC was incorrect",
                                                                        "You did not enter a different PIN",
                                                                        "That PIN is not long enough",
                                                                        "That PIN does not meet the complexity requirements",
                                                                        "Your PIN has expired and your account has become locked. You must contact support in order to unlock your account.",
                                                                        "You are unable to change your PIN because it has expired. Please contact support to reset the PIN"
        };

        private const string STORAGE_FILE = "logintext.data";
        private const string CONTAINER_NAME = "text-entries";
        private const string ENTRY_NAME = "text";
        private const string NAME_ATTR = "name";

        private readonly Dictionary<string, string> entries = new Dictionary<string, string>();

        public bool LoadFailed { get; private set; }
        public string FailReason { get; private set; }

        public string Username {
            get { return entries[USERNAME]; }
            set { entries[USERNAME] = value; }
        }

        public string Password {
            get { return entries[PASSWORD]; }
            set { entries[PASSWORD] = value; }
        }

        public string Otc {
            get { return entries[OTC]; }
            set { entries[OTC] = value; }
        }

        public string ShowImageButton {
            get { return entries[SHOW_IMAGE]; }
            set { entries[SHOW_IMAGE] = value; }
        }

        public string LoginButton {
            get { return entries[LOGIN]; }
            set { entries[LOGIN] = value; }
        }

        public string LogoFile {
            get { return entries[LOGO]; }
            set { entries[LOGO] = value; }
        }

        public string PageTitle {
            get { return entries[PAGE_TITLE]; }
            set { entries[PAGE_TITLE] = value; }
        }

        public string PageHeader {
            get { return entries[PAGE_HEADER]; }
            set { entries[PAGE_HEADER] = value; }
        }

        public string PageFooter {
            get { return entries[PAGE_FOOTER]; }
            set { entries[PAGE_FOOTER] = value; }
        }

        public string PINsafeLoginError {
            get { return entries[PINSAFE_FAILED]; }
            set { entries[PINSAFE_FAILED] = value; }
        }

        public string SharePointLoginError {
            get { return entries[SHAREPOINT_FAILED]; }
            set { entries[SHAREPOINT_FAILED] = value; }
        }

        public string OldOtc {
            get { return entries[OLD_OTC]; }
            set { entries[OLD_OTC] = value; }
        }

        public string NewOtc1 {
            get { return entries[NEW_OTC1]; }
            set { entries[NEW_OTC1] = value; }
        }

        public string NewOtc2 {
            get { return entries[NEW_OTC2]; }
            set { entries[NEW_OTC2] = value; }
        }

        public string ChangePinButton {
            get { return entries[CHANGE_PIN]; }
            set { entries[CHANGE_PIN] = value; }
        }

        public string ChangePinPageTitle {
            get { return entries[CHANGE_PIN_PAGE_TITLE]; }
            set { entries[CHANGE_PIN_PAGE_TITLE] = value; }
        }

        public string ChangePinPageHeader {
            get { return entries[CHANGE_PIN_PAGE_HEADER]; }
            set { entries[CHANGE_PIN_PAGE_HEADER] = value; }
        }

        public string ChangePinPageFooter {
            get { return entries[CHANGE_PIN_PAGE_FOOTER]; }
            set { entries[CHANGE_PIN_PAGE_FOOTER] = value; }
        }

        public string ChangePinError {
            get { return entries[CHANGE_PIN_FAILED]; }
            set { entries[CHANGE_PIN_FAILED] = value; }
        }

        public string OldPassword {
            get { return entries[OLD_PASSWORD]; }
            set { entries[OLD_PASSWORD] = value; }
        }

        public string NewPassword1 {
            get { return entries[NEW_PASSWORD1]; }
            set { entries[NEW_PASSWORD1] = value; }
        }

        public string NewPassword2 {
            get { return entries[NEW_PASSWORD2]; }
            set { entries[NEW_PASSWORD2] = value; }
        }

        public string ChangePasswordButton {
            get { return entries[CHANGE_PASSWORD]; }
            set { entries[CHANGE_PASSWORD] = value; }
        }

        public string ChangePasswordPageTitle {
            get { return entries[CHANGE_PASSWORD_PAGE_TITLE]; }
            set { entries[CHANGE_PASSWORD_PAGE_TITLE] = value; }
        }

        public string ChangePasswordPageHeader {
            get { return entries[CHANGE_PASSWORD_PAGE_HEADER]; }
            set { entries[CHANGE_PASSWORD_PAGE_HEADER] = value; }
        }

        public string ChangePasswordPageFooter {
            get { return entries[CHANGE_PASSWORD_PAGE_FOOTER]; }
            set { entries[CHANGE_PASSWORD_PAGE_FOOTER] = value; }
        }

        public string ChangePasswordError {
            get { return entries[CHANGE_PASSWORD_FAILED]; }
            set { entries[CHANGE_PASSWORD_FAILED] = value; }
        }

		public string ChangePasswordDisabled {
			get { return entries[CHANGE_PASSWORD_DISABLED]; }
			set { entries[CHANGE_PASSWORD_DISABLED] = value; }
		}

        public string UserNotFound {
            get { return entries[USER_UNKNOWN]; }
            set { entries[USER_UNKNOWN] = value; }
        }

        public string IncorrectOldPassword {
            get { return entries[INCORRECT_PASSWORD]; }
            set { entries[INCORRECT_PASSWORD] = value; }
        }

        public string PasswordPolicyError {
            get { return entries[PASSWORD_POLICY_ERROR]; }
            set { entries[PASSWORD_POLICY_ERROR] = value; }
        }

        public string UsernameMissing {
            get { return entries[USERNAME_MISSING]; }
            set { entries[USERNAME_MISSING] = value; }
        }

        public string PasswordMissing {
            get { return entries[PASSWORD_MISSING]; }
            set { entries[PASSWORD_MISSING] = value; }
        }

        public string OtcMissing {
            get { return entries[OTC_MISSING]; }
            set { entries[OTC_MISSING] = value; }
        }

	    public string ResetCode {
			get { return entries[RESET_CODE]; }
			set { entries[RESET_CODE] = value; }
	    }

		public string GetResetCodeButton {
			get { return entries[RESET_CODE_BUTTON]; }
			set { entries[RESET_CODE_BUTTON] = value; }
		}

		public string ResetButton {
			get { return entries[RESET_BUTTON]; }
			set { entries[RESET_BUTTON] = value; }
		}

		public string ResetPageTitle {
			get { return entries[RESET_PAGE_TITLE]; }
			set { entries[RESET_PAGE_TITLE] = value; }
		}

		public string ResetPageHeader {
			get { return entries[RESET_PAGE_HEADER]; }
			set { entries[RESET_PAGE_HEADER] = value; }
		}

		public string ResetPageFooter {
			get { return entries[RESET_PAGE_FOOTER]; }
			set { entries[RESET_PAGE_FOOTER] = value; }
		}

	    public string ResetPinSuccess {
			get { return entries[RESET_PIN_SUCCESS]; }
			set { entries[RESET_PIN_SUCCESS] = value; }
	    }

		public string ResetPasswordSuccess {
			get { return entries[RESET_PASSWORD_SUCCESS]; }
			set { entries[RESET_PASSWORD_SUCCESS] = value; }
		}

		public string ResetPinFailed {
			get { return entries[RESET_PIN_FAILED]; }
			set { entries[RESET_PIN_FAILED] = value; }
		}

		public string ResetPasswordFailed {
			get { return entries[RESET_PASSWORD_FAILED]; }
			set { entries[RESET_PASSWORD_FAILED] = value; }
		}

	    public string ResetPasswordAlert {
			get { return entries[RESET_PWD_ALERT]; }
			set { entries[RESET_PWD_ALERT] = value; }
	    }

	    public string ForgottenPin {
			get { return entries[FORGOTTEN_PIN]; }
			set { entries[FORGOTTEN_PIN] = value; }
	    }

		public string ForgottenPassword {
			get { return entries[FORGOTTEN_PASSWORD]; }
			set { entries[FORGOTTEN_PASSWORD] = value; }
		}

		public string ForgottenPinOrPassword {
			get { return entries[FORGOTTEN_PIN_PASSWORD]; }
			set { entries[FORGOTTEN_PIN_PASSWORD] = value; }
		}

        public string GetText(string message) {
            if (string.IsNullOrEmpty(message)) {
                return "";
            }
            string code = "__" + message;
            return (entries.ContainsKey(code)) ? entries[code] : message;
        }

        public Dictionary<string, string> CustomStrings {
            get {
                Dictionary<string, string> strings = new Dictionary<string, string>();
                foreach (KeyValuePair<string, string> entry in entries.Where(entry => entry.Key.StartsWith("__"))) {
                    strings[entry.Key.Substring(2)] = entry.Value;
                }
                return strings;
            }
            set {
                HashSet<string> deleted = new HashSet<string>();
                foreach (KeyValuePair<string, string> entry in entries.Where(entry => entry.Key.StartsWith("__"))) {
                    deleted.Add(entry.Key);
                }
                foreach (string code in deleted) {
                    entries.Remove(code);
                }
                foreach (KeyValuePair<string, string> entry in value.Where(entry => !string.IsNullOrEmpty(entry.Key))) {
                    entries["__" + entry.Key] = entry.Value;
                }
            }
        }

	    public string ResetDisabled {
		    get { return entries[RESET_DISABLED]; }
			set { entries[RESET_DISABLED] = value; }
	    }

	    public TextSettings() {
            LoadFromStorage();
        }

        public void LoadFromStorage() {
            entries.Clear();
            XmlDocument doc = new XmlDocument();
            if (PINsafeSettings.ConfigFileExists(STORAGE_FILE)) {
                using (Stream stream = PINsafeSettings.OpenConfigFileForRead(STORAGE_FILE)) {
                    if (stream == null) {
                        LoadFailed = true;
                        FailReason = PINsafeSettings.FailReason;
                        return;
                    }
                    try {
                        doc.Load(stream);
                        XmlElement root = doc.DocumentElement;
                        if (root != null) {
                            XmlNodeList elems = root.GetElementsByTagName(ENTRY_NAME);
                            foreach (XmlElement elem in elems) {
                                string name = elem.GetAttribute(NAME_ATTR);
                                if (!"".Equals(name)) {
                                    entries[name] = elem.InnerText;
                                }
                            }
                        }
                    } catch (XmlException e) {
                        LoadFailed = true;
                        FailReason = e.Message;
                    }
                }
                FillDefaults();
            } else {
                LoadFailed = true;
                FailReason = "File not found";
                SetDefaults();
            }
        }

        public bool LoadFromCentralConfig(string serverName) {
            XmlDocument doc = new XmlDocument();
            if (!PINsafeSettings.SharedConfigFileExists(serverName, STORAGE_FILE)) {
                return false;
            }
            using (Stream stream = PINsafeSettings.OpenSharedConfig(serverName, STORAGE_FILE)) {
                try {
                    doc.Load(stream);
                    XmlElement root = doc.DocumentElement;
                    if (root != null) {
                        XmlNodeList elems = root.GetElementsByTagName(ENTRY_NAME);
                        foreach (XmlElement elem in elems) {
                            string name = elem.GetAttribute(NAME_ATTR);
                            if (!"".Equals(name)) {
                                entries[name] = elem.InnerText;
                            }
                        }
                    }
                } catch {
                    return false;
                }
            }
            return true;
        }

        public void SetDefaults() {
            entries.Clear();
            FillDefaults();
            SaveToStorage();
        }

        private void FillDefaults() {
            SetIfEmpty(USERNAME, DEFAULT_USERNAME);
            SetIfEmpty(PASSWORD, DEFAULT_PASSWORD);
            SetIfEmpty(OTC, DEFAULT_OTC);
            SetIfEmpty(SHOW_IMAGE, DEFAULT_SHOW_IMAGE);
            SetIfEmpty(LOGIN, DEFAULT_LOGIN);
            SetIfEmpty(LOGO, DEFAULT_LOGO);
            SetIfEmpty(PAGE_TITLE, DEFAULT_PAGE_TITLE);
            SetIfEmpty(PAGE_HEADER, DEFAULT_PAGE_HEADER);
            SetIfEmpty(PAGE_FOOTER, DEFAULT_PAGE_FOOTER);
            SetIfEmpty(PINSAFE_FAILED, DEFAULT_PINSAFE_FAILED);
            SetIfEmpty(SHAREPOINT_FAILED, DEFAULT_SHAREPOINT_FAILED);
            SetIfEmpty(OLD_OTC, DEFAULT_OLD_OTC);
            SetIfEmpty(NEW_OTC1, DEFAULT_NEW_OTC1);
            SetIfEmpty(NEW_OTC2, DEFAULT_NEW_OTC2);
            SetIfEmpty(CHANGE_PIN, DEFAULT_CHANGE_PIN);
            SetIfEmpty(CHANGE_PIN_PAGE_TITLE, DEFAULT_CHANGE_PIN_PAGE_TITLE);
            SetIfEmpty(CHANGE_PIN_PAGE_HEADER, DEFAULT_CHANGE_PIN_PAGE_HEADER);
            SetIfEmpty(CHANGE_PIN_PAGE_FOOTER, DEFAULT_CHANGE_PIN_PAGE_FOOTER);
            SetIfEmpty(CHANGE_PIN_FAILED, DEFAULT_CHANGE_PIN_FAILED);
            SetIfEmpty(OLD_PASSWORD, DEFAULT_OLD_PASSWORD);
            SetIfEmpty(NEW_PASSWORD1, DEFAULT_NEW_PASSWORD1);
            SetIfEmpty(NEW_PASSWORD2, DEFAULT_NEW_PASSWORD2);
            SetIfEmpty(CHANGE_PASSWORD, DEFAULT_CHANGE_PASSWORD);
            SetIfEmpty(CHANGE_PASSWORD_PAGE_TITLE, DEFAULT_CHANGE_PASSWORD_PAGE_TITLE);
            SetIfEmpty(CHANGE_PASSWORD_PAGE_HEADER, DEFAULT_CHANGE_PASSWORD_PAGE_HEADER);
            SetIfEmpty(CHANGE_PASSWORD_PAGE_FOOTER, DEFAULT_CHANGE_PASSWORD_PAGE_FOOTER);
            SetIfEmpty(CHANGE_PASSWORD_FAILED, DEFAULT_CHANGE_PASSWORD_FAILED);
			SetIfEmpty(CHANGE_PASSWORD_DISABLED, DEFAULT_CHANGE_PASSWORD_DISABLED);
			SetIfEmpty(USER_UNKNOWN, DEFAULT_USER_UNKNOWN);
            SetIfEmpty(INCORRECT_PASSWORD, DEFAULT_INCORRECT_PASSWORD);
            SetIfEmpty(PASSWORD_POLICY_ERROR, DEFAULT_PASSWORD_POLICY_ERROR);
            SetIfEmpty(USERNAME_MISSING, DEFAULT_USERNAME_MISSING);
            SetIfEmpty(PASSWORD_MISSING, DEFAULT_PASSWORD_MISSING);
            SetIfEmpty(OTC_MISSING, DEFAULT_OTC_MISSING);
			SetIfEmpty(RESET_CODE, DEFAULT_RESET_CODE);
			SetIfEmpty(RESET_CODE_BUTTON, DEFAULT_RESET_CODE_BUTTON);
			SetIfEmpty(RESET_BUTTON, DEFAULT_RESET_BUTTON);
			SetIfEmpty(RESET_PAGE_TITLE, DEFAULT_RESET_PAGE_TITLE);
			SetIfEmpty(RESET_PAGE_HEADER, DEFAULT_RESET_PAGE_HEADER);
			SetIfEmpty(RESET_PAGE_FOOTER, DEFAULT_RESET_PAGE_FOOTER);
			SetIfEmpty(RESET_PIN_SUCCESS, DEFAULT_RESET_PIN_SUCCESS);
			SetIfEmpty(RESET_PASSWORD_SUCCESS, DEFAULT_RESET_PASSWORD_SUCCESS);
			SetIfEmpty(RESET_PIN_FAILED, DEFAULT_RESET_PIN_FAILED);
			SetIfEmpty(RESET_PASSWORD_FAILED, DEFAULT_RESET_PASSWORD_FAILED);
			SetIfEmpty(RESET_PWD_ALERT, DEFAULT_RESET_PWD_ALERT);
			SetIfEmpty(RESET_DISABLED, DEFAULT_RESET_DISABLED);
			SetIfEmpty(FORGOTTEN_PIN, DEFAULT_FORGOTTEN_PIN);
			SetIfEmpty(FORGOTTEN_PASSWORD, DEFAULT_FORGOTTEN_PASSWORD);
			SetIfEmpty(FORGOTTEN_PIN_PASSWORD, DEFAULT_FORGOTTEN_PIN_PASSWORD);

            // Default custom error strings
            for (int idx = 0; idx < CUSTOM_ERROR_CODES.Length; idx++) {
                string code = CUSTOM_ERROR_CODES[idx];
                if (!entries.ContainsKey(code)) {
                    entries[code] = CUSTOM_ERROR_MESSAGES[idx];
                }
            }
        }

        private void SetIfEmpty(string name, string value) {
            if (!entries.ContainsKey(name)) {
                entries[name] = value;
            }
        }

        public bool SaveToStorage() {
            try {
                XmlDocument doc = new XmlDocument();
                XmlElement root = doc.CreateElement(CONTAINER_NAME);
                doc.AppendChild(root);
                foreach (KeyValuePair<string, string> entry in entries) {
                    XmlElement elem = doc.CreateElement(ENTRY_NAME);
                    elem.SetAttribute(NAME_ATTR, entry.Key);
                    elem.InnerText = entry.Value;
                    root.AppendChild(elem);
                }
                using (Stream stream = PINsafeSettings.OpenConfigFileForWrite(STORAGE_FILE)) {
                    doc.Save(stream);
                }
                return true;
            } catch {
                return false;
            }
        }
    }
}
