﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using com.swivelsecure.pinsafe.config.Properties;

namespace com.swivelsecure.pinsafe.config
{
    public partial class ConfigForm : Form {
        private readonly PINsafeSettings pinsafeSettings = PINsafeSettings.EmptySettings;
        private readonly SharePointSettings sharePointSettings = new SharePointSettings();
        private readonly TextSettings textSettings = new TextSettings();

        public ConfigForm() {
            InitializeComponent();
            RefreshSettings();
        }

        private void SettingsToDisplay() {
            // PINsafe settings
            txtServer.Text = pinsafeSettings.Server;
            txtPort.Text = pinsafeSettings.Port.ToString(CultureInfo.InvariantCulture);
            txtContext.Text = pinsafeSettings.Context;
            txtSecret.Text = pinsafeSettings.Secret;
            txtSecret2.Text = pinsafeSettings.Secret;
            drpSecure.SelectedIndex = (pinsafeSettings.Ssl ? 1 : 0);
            chkSelfSigned.Checked = pinsafeSettings.AcceptSelfSigned;
            if (pinsafeSettings.ShowTURing) {
                rbTuring.Checked = true;
            } else if (pinsafeSettings.ShowMessage) {
                rbMessage.Checked = true;
            } else {
                rbNoImage.Checked = true;
            }
            if (pinsafeSettings.AllowNonPINsafeUsers) {
                if (string.IsNullOrEmpty(pinsafeSettings.RedirectUrl)) {
                    drpUnknownUsers.SelectedIndex = 1;
                    lblRedirectUrl.Enabled = false;
                    txtRedirectUrl.Enabled = false;
                } else {
                    drpUnknownUsers.SelectedIndex = 2;
                    lblRedirectUrl.Enabled = true;
                    txtRedirectUrl.Enabled = true;
                    txtRedirectUrl.Text = pinsafeSettings.RedirectUrl;
                }
            } else {
                drpUnknownUsers.SelectedIndex = 0;
                lblRedirectUrl.Enabled = false;
                txtRedirectUrl.Enabled = false;
            }
	        chkDebugEnabled.Checked = pinsafeSettings.DebugEnabled;

            // Authentication settings
            txtAuthDomain.Text = pinsafeSettings.AuthenticationDomain;
            txtLogonPath.Text = pinsafeSettings.LogonUrl;
            txtChangePinPath.Text = pinsafeSettings.ChangePinUrl;
            txtChangePwdPath.Text = pinsafeSettings.ChangePasswordUrl;
            txtAuthPath.Text = pinsafeSettings.AuthUrl;
            txtWebApp.Text = pinsafeSettings.WebApplication;
            txtExcPaths.Lines = pinsafeSettings.ExcludedPaths;

            // Sharepoint settings
            txtProviderName.Text = sharePointSettings.ProviderName;
            txtConnectionStringName.Text = sharePointSettings.ConnectionStringName;
            txtConnectionString.Text = sharePointSettings.ConnectionString;
            txtUsernameAttr.Text = sharePointSettings.UsernameAttr;
            lblCentralPath.Text = sharePointSettings.CentralAdminPath;
            chkCentral.Checked = sharePointSettings.CentralAdminExists;
            lblTokenPath.Text = sharePointSettings.SecurityTokenPath;
            chkToken.Checked = sharePointSettings.SecurityTokenExists;

            // Applications
            lstApps.Items.Clear();
            lstApps.DisplayMember = "Display";
            foreach (SharePointSettings.SharePointApplication app in sharePointSettings.GetLocations()) {
                lstApps.Items.Add(app);
            }

            // Login Page Text
            txtUsernameText.Text = textSettings.Username;
            txtPasswordText.Text = textSettings.Password;
            txtOTCText.Text = textSettings.Otc;
            txtImageBtnText.Text = textSettings.ShowImageButton;
            txtLoginBtnText.Text = textSettings.LoginButton;
            txtLogoFile.Text = textSettings.LogoFile;
            txtTitleText.Text = textSettings.PageTitle;
            txtHeaderText.Text = textSettings.PageHeader;
            txtFooterText.Text = textSettings.PageFooter;
	        txtForgottenPinPwd.Text = textSettings.ForgottenPinOrPassword;
            txtPINsafeFailedText.Text = textSettings.PINsafeLoginError;
            txtSharePointError.Text = textSettings.SharePointLoginError;

            // Change PIN Page Text
            txtOldOtc.Text = textSettings.OldOtc;
            txtNewOtc1.Text = textSettings.NewOtc1;
            txtNewOtc2.Text = textSettings.NewOtc2;
            txtChangePinBtn.Text = textSettings.ChangePinButton;
            txtChangePinTitle.Text = textSettings.ChangePinPageTitle;
            txtChangePinHeader.Text = textSettings.ChangePinPageHeader;
            txtChangePinFooter.Text = textSettings.ChangePinPageFooter;
	        txtForgottenPin.Text = textSettings.ForgottenPin;
            txtChangePinFailed.Text = textSettings.ChangePinError;

            // Change Password Page Text
            txtOldPassword.Text = textSettings.OldPassword;
            txtNewPassword1.Text = textSettings.NewPassword1;
            txtNewPassword2.Text = textSettings.NewPassword2;
            txtChangePasswordBtn.Text = textSettings.ChangePasswordButton;
            txtChangePasswordTitle.Text = textSettings.ChangePasswordPageTitle;
            txtChangePasswordHeader.Text = textSettings.ChangePasswordPageHeader;
            txtChangePasswordFooter.Text = textSettings.ChangePasswordPageFooter;
	        txtForgottenPassword.Text = textSettings.ForgottenPassword;
            txtChangePasswordFailed.Text = textSettings.ChangePasswordError;
			txtChangePasswordDisabled.Text = textSettings.ChangePasswordDisabled;
			txtWrongPassword.Text = textSettings.IncorrectOldPassword;
            txtPwdPolicyError.Text = textSettings.PasswordPolicyError;
            txtUnknownUser.Text = textSettings.UserNotFound;
            
            // Change Password settings
	        bool pwdsEnabled = pinsafeSettings.ManageADPasswords;
	        chkManagePasswords.Checked = pwdsEnabled;
	        drpUsernameType.Enabled = pwdsEnabled;
	        txtUserAccount.Enabled = pwdsEnabled;
	        txtUserPassword1.Enabled = pwdsEnabled;
	        txtUserPassword2.Enabled = pwdsEnabled;
            drpUsernameType.SelectedIndex = (pinsafeSettings.UsernameType ==
                                             PINsafeSettings.UsernameTypes.sAMAccountName)
                                                ? 0
                                                : 1;
            txtUserAccount.Text = pinsafeSettings.UserAccount;

			// Reset Page settings/text
	        chkPinReset.Checked = pinsafeSettings.ResetPinEnabled;
			chkPwdReset.Enabled = pwdsEnabled && pinsafeSettings.ResetPinEnabled;
			chkPwdReset.Checked = chkPwdReset.Enabled && pinsafeSettings.ResetPasswordEnabled;
	        txtResetCodeText.Text = textSettings.ResetCode;
	        txtResetCodeButton.Text = textSettings.GetResetCodeButton;
	        txtResetButton.Text = textSettings.ResetButton;
	        txtResetPageTitle.Text = textSettings.ResetPageTitle;
	        txtResetPageHeader.Text = textSettings.ResetPageHeader;
	        txtResetPageFooter.Text = textSettings.ResetPageFooter;
	        txtResetPinSuccess.Text = textSettings.ResetPinSuccess;
			txtResetPwdSuccess.Text = textSettings.ResetPasswordSuccess;
			txtResetPinFailed.Text = textSettings.ResetPinFailed;
			txtResetPwdFailed.Text = textSettings.ResetPasswordFailed;
	        txtResetAlert.Text = textSettings.ResetPasswordAlert;
	        txtResetDisabled.Text = textSettings.ResetDisabled;

            // Custom strings
            Dictionary<string, string> customStrings = textSettings.CustomStrings;
            lstErrors.Items.Clear();
            foreach (KeyValuePair<string, string> item in customStrings) {
                ListViewItem lvItem = new ListViewItem(item.Key);
                lvItem.SubItems.Add(item.Value);
                lstErrors.Items.Add(lvItem);
            }
        }

        private bool DisplayToPINsafeSettings() {
            // Check all values against current settings, and update if necessary.
            bool changed = false;

            // PINsafe settings
            if (String.CompareOrdinal(txtServer.Text, pinsafeSettings.Server) != 0) {
                pinsafeSettings.Server = txtServer.Text;
                changed = true;
            }
            int newPort;
            if (int.TryParse(txtPort.Text, out newPort)) {
                if (newPort != pinsafeSettings.Port) {
                    pinsafeSettings.Port = newPort;
                    changed = true;
                }
            }

            if (String.CompareOrdinal(txtContext.Text, pinsafeSettings.Context) != 0) {
                pinsafeSettings.Context = txtContext.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtSecret.Text, pinsafeSettings.Secret) != 0) {
                pinsafeSettings.Secret = txtSecret.Text;
                changed = true;
            }

            if ((drpSecure.SelectedIndex == 0 && pinsafeSettings.Ssl) || (drpSecure.SelectedIndex == 1 && !pinsafeSettings.Ssl)) {
                pinsafeSettings.Ssl = (drpSecure.SelectedIndex == 1);
                changed = true;
            }

            if (chkSelfSigned.Checked != pinsafeSettings.AcceptSelfSigned) {
                pinsafeSettings.AcceptSelfSigned = chkSelfSigned.Checked;
                changed = true;
            }

            if (rbTuring.Checked != pinsafeSettings.ShowTURing) {
                pinsafeSettings.ShowTURing = rbTuring.Checked;
                changed = true;
            }

            if (rbMessage.Checked != pinsafeSettings.ShowMessage) {
                pinsafeSettings.ShowMessage = rbMessage.Checked;
                changed = true;
            }

	        if (pinsafeSettings.DebugEnabled != chkDebugEnabled.Checked) {
		        pinsafeSettings.DebugEnabled = chkDebugEnabled.Checked;
		        changed = true;
	        }

	        // Authentication settings
            switch (drpUnknownUsers.SelectedIndex) {
            case 1: // Allow unknown users with password only.
                if (!pinsafeSettings.AllowNonPINsafeUsers) {
                    pinsafeSettings.AllowNonPINsafeUsers = true;
                    changed = true;
                }
                if (!string.IsNullOrEmpty(pinsafeSettings.RedirectUrl)) {
                    pinsafeSettings.RedirectUrl = "";
                    changed = true;
                }
                break;

            case 2:
                if (!pinsafeSettings.AllowNonPINsafeUsers) {
                    pinsafeSettings.AllowNonPINsafeUsers = true;
                    pinsafeSettings.RedirectUrl = txtRedirectUrl.Text;
                    changed = true;
                } else if (pinsafeSettings.RedirectUrl == null || String.Compare(pinsafeSettings.RedirectUrl, txtRedirectUrl.Text, StringComparison.OrdinalIgnoreCase) != 0) {
                    pinsafeSettings.RedirectUrl = txtRedirectUrl.Text;
                    changed = true;
                }
                break;

            default:
                if (pinsafeSettings.AllowNonPINsafeUsers) {
                    pinsafeSettings.AllowNonPINsafeUsers = false;
                    pinsafeSettings.RedirectUrl = "";
                    changed = true;
                }
                break;
            }

            if (String.CompareOrdinal(txtAuthDomain.Text, pinsafeSettings.AuthenticationDomain) != 0) {
                pinsafeSettings.AuthenticationDomain = txtAuthDomain.Text;
                changed = true;
            }
            if (String.CompareOrdinal(txtLogonPath.Text, pinsafeSettings.LogonUrl) != 0) {
                pinsafeSettings.LogonUrl = txtLogonPath.Text;
                changed = true;
            }
            if (String.CompareOrdinal(txtChangePinPath.Text, pinsafeSettings.ChangePinUrl) != 0) {
                pinsafeSettings.ChangePinUrl = txtChangePinPath.Text;
                changed = true;
            }
            if (String.CompareOrdinal(txtChangePwdPath.Text, pinsafeSettings.ChangePasswordUrl) != 0) {
                pinsafeSettings.ChangePasswordUrl = txtChangePwdPath.Text;
                changed = true;
            }
            if (String.CompareOrdinal(txtAuthPath.Text, pinsafeSettings.AuthUrl) != 0) {
                pinsafeSettings.AuthUrl = txtAuthPath.Text;
                changed = true;
            }
            if (String.CompareOrdinal(txtWebApp.Text, pinsafeSettings.WebApplication) != 0) {
                pinsafeSettings.WebApplication = txtWebApp.Text;
                changed = true;
            }
            string[] newExcPaths = txtExcPaths.Lines;
            string[] oldExcPaths = pinsafeSettings.ExcludedPaths;
            if (newExcPaths.Length != oldExcPaths.Length) {
                pinsafeSettings.ExcludedPaths = newExcPaths;
                changed = true;
            } else {
                if (newExcPaths.Where((t, idx) => String.CompareOrdinal(t, oldExcPaths[idx]) != 0).Any()) {
                    pinsafeSettings.ExcludedPaths = newExcPaths;
                    changed = true;
                }
            }
            if (drpUsernameType.SelectedIndex == 0 && pinsafeSettings.UsernameType != PINsafeSettings.UsernameTypes.sAMAccountName) {
                pinsafeSettings.UsernameType = PINsafeSettings.UsernameTypes.sAMAccountName;
                changed = true;
            } else if (drpUsernameType.SelectedIndex == 1 && pinsafeSettings.UsernameType != PINsafeSettings.UsernameTypes.userPrincipalName) {
                pinsafeSettings.UsernameType = PINsafeSettings.UsernameTypes.userPrincipalName;
                changed = true;
            }
            if (String.CompareOrdinal(txtUserAccount.Text, pinsafeSettings.UserAccount) != 0) {
                pinsafeSettings.UserAccount = txtUserAccount.Text;
                changed = true;
            }
            if (!String.IsNullOrEmpty(txtUserPassword1.Text)) {
                pinsafeSettings.UserPassword = txtUserPassword1.Text;
                changed = true;
                txtUserPassword1.Text = "";
                txtUserPassword2.Text = "";
            }

			if (chkPinReset.Checked != pinsafeSettings.ResetPinEnabled) {
				pinsafeSettings.ResetPinEnabled = chkPinReset.Checked;
				changed = true;
			}
			if (chkPwdReset.Checked != pinsafeSettings.ResetPasswordEnabled) {
				pinsafeSettings.ResetPasswordEnabled = chkPwdReset.Checked;
				changed = true;
			}
			if (chkManagePasswords.Checked != pinsafeSettings.ManageADPasswords) {
				pinsafeSettings.ManageADPasswords = chkManagePasswords.Checked;
				changed = true;
			}

			return changed;
        }

        private bool DisplayToSharePointSettings() {
            bool changed = false;

            if (String.CompareOrdinal(sharePointSettings.ProviderName, txtProviderName.Text) != 0) {
                sharePointSettings.OldProviderName = sharePointSettings.ProviderName;
                sharePointSettings.ProviderName = txtProviderName.Text;
                changed = true;
            }

            if (String.CompareOrdinal(sharePointSettings.ConnectionStringName, txtConnectionStringName.Text) != 0) {
                sharePointSettings.OldConnectionStringName = sharePointSettings.ConnectionStringName;
                sharePointSettings.ConnectionStringName = txtConnectionStringName.Text;
                changed = true;
            }

            if (String.CompareOrdinal(sharePointSettings.ConnectionString, txtConnectionString.Text) != 0) {
                sharePointSettings.ConnectionString = txtConnectionString.Text;
                changed = true;
            }

            if (String.CompareOrdinal(sharePointSettings.UsernameAttr, txtUsernameAttr.Text) != 0) {
                sharePointSettings.UsernameAttr = txtUsernameAttr.Text;
                changed = true;
            }

            if (String.CompareOrdinal(sharePointSettings.CentralAdminPath, lblCentralPath.Text) != 0) {
                sharePointSettings.CentralAdminPath = lblCentralPath.Text;
                changed = true;
            }

            if (chkCentral.Checked != sharePointSettings.CentralAdminExists) {
                sharePointSettings.CentralAdminExists = chkCentral.Checked;
                changed = true;
            }

            if (String.CompareOrdinal(sharePointSettings.SecurityTokenPath, lblTokenPath.Text) != 0) {
                sharePointSettings.SecurityTokenPath = lblTokenPath.Text;
                changed = true;
            }

            if (chkToken.Checked != sharePointSettings.SecurityTokenExists) {
                sharePointSettings.SecurityTokenExists = chkToken.Checked;
                changed = true;
            }

            return changed;
        }

        private bool DisplayToApplicationList() {
            List<SharePointSettings.SharePointApplication> current = new List<SharePointSettings.SharePointApplication>(lstApps.Items.Count);
            current.AddRange(sharePointSettings.GetLocations());
            List<SharePointSettings.SharePointApplication> deleted = (from app in current
                                    let found =
                                        lstApps.Items.Cast<SharePointSettings.SharePointApplication>().Any(
                                            item => String.Compare(app.Location, item.Location, StringComparison.OrdinalIgnoreCase) == 0)
                                    where !found
                                    select app).ToList();
            List<SharePointSettings.SharePointApplication> added = (from SharePointSettings.SharePointApplication item in lstApps.Items
                                  let found = current.Any(app => String.Compare(app.Location, item.Location, StringComparison.OrdinalIgnoreCase) == 0)
                                  where !found
                                  select item).ToList();
            if (deleted.Count == 0 && added.Count == 0) {
                return false;
            }
            foreach (SharePointSettings.SharePointApplication app in deleted) {
                sharePointSettings.RemoveLocation(app.Location);
            }
            foreach (SharePointSettings.SharePointApplication app in added) {
                sharePointSettings.AddLocation(app.Location, app.RequiresPINsafe);
            }
            return true;
        }

        private bool DisplayToTextSettings() {
            bool changed = false;

            if (String.CompareOrdinal(txtUsernameText.Text, textSettings.Username) != 0) {
                textSettings.Username = txtUsernameText.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtPasswordText.Text, textSettings.Password) != 0) {
                textSettings.Password = txtPasswordText.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtOTCText.Text, textSettings.Otc) != 0) {
                textSettings.Otc = txtOTCText.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtLoginBtnText.Text, textSettings.LoginButton) != 0) {
                textSettings.LoginButton = txtLoginBtnText.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtImageBtnText.Text, textSettings.ShowImageButton) != 0) {
                textSettings.ShowImageButton = txtImageBtnText.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtLogoFile.Text, textSettings.LogoFile) != 0) {
                textSettings.LogoFile = txtLogoFile.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtTitleText.Text, textSettings.PageTitle) != 0) {
                textSettings.PageTitle = txtTitleText.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtHeaderText.Text, textSettings.PageHeader) != 0) {
                textSettings.PageHeader = txtHeaderText.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtFooterText.Text, textSettings.PageFooter) != 0) {
                textSettings.PageFooter = txtFooterText.Text;
                changed = true;
            }

			if (String.CompareOrdinal(txtForgottenPinPwd.Text, textSettings.ForgottenPinOrPassword) != 0) {
				textSettings.ForgottenPinOrPassword = txtForgottenPinPwd.Text;
				changed = true;
			}

            if (String.CompareOrdinal(txtPINsafeFailedText.Text, textSettings.PINsafeLoginError) != 0) {
                textSettings.PINsafeLoginError = txtPINsafeFailedText.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtSharePointError.Text, textSettings.SharePointLoginError) != 0) {
                textSettings.SharePointLoginError = txtSharePointError.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtOldOtc.Text, textSettings.OldOtc) != 0) {
                textSettings.OldOtc = txtOldOtc.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtNewOtc1.Text, textSettings.NewOtc1) != 0) {
                textSettings.NewOtc1 = txtNewOtc1.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtNewOtc2.Text, textSettings.NewOtc2) != 0) {
                textSettings.NewOtc2 = txtNewOtc2.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtChangePinBtn.Text, textSettings.ChangePinButton) != 0) {
                textSettings.ChangePinButton = txtChangePinBtn.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtChangePinTitle.Text, textSettings.ChangePinPageTitle) != 0) {
                textSettings.ChangePinPageTitle = txtChangePinTitle.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtChangePinHeader.Text, textSettings.ChangePinPageHeader) != 0) {
                textSettings.ChangePinPageHeader = txtChangePinHeader.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtChangePinFooter.Text, textSettings.ChangePinPageFooter) != 0) {
                textSettings.ChangePinPageFooter = txtChangePinFooter.Text;
                changed = true;
            }

			if (String.CompareOrdinal(txtForgottenPin.Text, textSettings.ForgottenPin) != 0) {
				textSettings.ForgottenPin = txtForgottenPin.Text;
				changed = true;
			}

            if (String.CompareOrdinal(txtChangePinFailed.Text, textSettings.ChangePinError) != 0) {
                textSettings.ChangePinError = txtChangePinFailed.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtOldPassword.Text, textSettings.OldPassword) != 0) {
                textSettings.OldPassword = txtOldPassword.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtNewPassword1.Text, textSettings.NewPassword1) != 0) {
                textSettings.NewPassword1 = txtNewPassword1.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtNewPassword2.Text, textSettings.NewPassword2) != 0) {
                textSettings.NewPassword2 = txtNewPassword2.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtChangePasswordBtn.Text, textSettings.ChangePasswordButton) != 0) {
                textSettings.ChangePasswordButton = txtChangePasswordBtn.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtChangePasswordTitle.Text, textSettings.ChangePasswordPageTitle) != 0) {
                textSettings.ChangePasswordPageTitle = txtChangePasswordTitle.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtChangePasswordHeader.Text, textSettings.ChangePasswordPageHeader) != 0) {
                textSettings.ChangePasswordPageHeader = txtChangePasswordHeader.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtChangePasswordFooter.Text, textSettings.ChangePasswordPageFooter) != 0) {
                textSettings.ChangePasswordPageFooter = txtChangePasswordFooter.Text;
                changed = true;
            }

			if (String.CompareOrdinal(txtForgottenPassword.Text, textSettings.ForgottenPassword) != 0) {
				textSettings.ForgottenPassword = txtForgottenPassword.Text;
				changed = true;
			}

            if (String.CompareOrdinal(txtChangePasswordFailed.Text, textSettings.ChangePasswordError) != 0) {
                textSettings.ChangePasswordError = txtChangePasswordFailed.Text;
                changed = true;
            }

			if (String.CompareOrdinal(txtChangePasswordDisabled.Text, textSettings.ChangePasswordDisabled) != 0) {
				textSettings.ChangePasswordDisabled = txtChangePasswordDisabled.Text;
				changed = true;
			}

            if (String.CompareOrdinal(txtWrongPassword.Text, textSettings.IncorrectOldPassword) != 0) {
                textSettings.IncorrectOldPassword = txtWrongPassword.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtPwdPolicyError.Text, textSettings.PasswordPolicyError) != 0) {
                textSettings.PasswordPolicyError = txtPwdPolicyError.Text;
                changed = true;
            }

            if (String.CompareOrdinal(txtUnknownUser.Text, textSettings.UserNotFound) != 0) {
                textSettings.UserNotFound = txtUnknownUser.Text;
                changed = true;
            }

			// Reset Page
			if (String.CompareOrdinal(txtResetCodeText.Text, textSettings.ResetCode) != 0) {
				textSettings.ResetCode = txtResetCodeText.Text;
				changed = true;
			}

			if (String.CompareOrdinal(txtResetCodeButton.Text, textSettings.GetResetCodeButton) != 0) {
				textSettings.GetResetCodeButton = txtResetCodeButton.Text;
				changed = true;
			}

			if (String.CompareOrdinal(txtResetButton.Text, textSettings.ResetButton) != 0) {
				textSettings.ResetButton = txtResetButton.Text;
				changed = true;
			}

			if (String.CompareOrdinal(txtResetPageTitle.Text, textSettings.ResetPageTitle) != 0) {
				textSettings.ResetPageTitle = txtResetPageTitle.Text;
				changed = true;
			}

			if (String.CompareOrdinal(txtResetPageHeader.Text, textSettings.ResetPageHeader) != 0) {
				textSettings.ResetPageHeader = txtResetPageHeader.Text;
				changed = true;
			}

			if (String.CompareOrdinal(txtResetPageFooter.Text, textSettings.ResetPageFooter) != 0) {
				textSettings.ResetPageFooter = txtResetPageFooter.Text;
				changed = true;
			}

			if (String.CompareOrdinal(txtResetPinSuccess.Text, textSettings.ResetPinSuccess) != 0) {
				textSettings.ResetPinSuccess = txtResetPinSuccess.Text;
				changed = true;
			}

			if (String.CompareOrdinal(txtResetPwdSuccess.Text, textSettings.ResetPasswordSuccess) != 0) {
				textSettings.ResetPasswordSuccess = txtResetPwdSuccess.Text;
				changed = true;
			}

			if (String.CompareOrdinal(txtResetPinFailed.Text, textSettings.ResetPinFailed) != 0) {
				textSettings.ResetPinFailed = txtResetPinFailed.Text;
				changed = true;
			}

			if (String.CompareOrdinal(txtResetPwdFailed.Text, textSettings.ResetPasswordFailed) != 0) {
				textSettings.ResetPasswordFailed = txtResetPwdFailed.Text;
				changed = true;
			}

			if (String.CompareOrdinal(txtResetAlert.Text, textSettings.ResetPasswordAlert) != 0) {
				textSettings.ResetPasswordAlert = txtResetAlert.Text;
				changed = true;
			}

			if (String.CompareOrdinal(txtResetDisabled.Text, textSettings.ResetDisabled) != 0) {
				textSettings.ResetDisabled = txtResetDisabled.Text;
				changed = true;
			}

			// Custom strings
            Dictionary<string, string> customStrings = textSettings.CustomStrings;
            List<string> keys = customStrings.Keys.ToList();
            foreach (ListViewItem lvItem in lstErrors.Items) {
                string key = lvItem.Text;
                if (customStrings.ContainsKey(key)) {
                    keys.Remove(key);
                    string value = lvItem.SubItems[1].Text;
                    if (string.Compare(value, customStrings[key], StringComparison.Ordinal) != 0) {
                        customStrings[key] = value;
                        changed = true;
                    }
                } else {
                    customStrings.Add(key, lvItem.SubItems[1].Text);
                    changed = true;
                }
            }
            // Remove deleted items
            if (keys.Count > 0) {
                changed = true;
                foreach (string key in keys) {
                    customStrings.Remove(key);
                }
            }
            if (changed) {
                textSettings.CustomStrings = customStrings;
            }

            return changed;
        }

        private bool CheckSecret() {
            if (String.CompareOrdinal(txtSecret.Text, pinsafeSettings.Secret) == 0) {
                // Value hasn't changed - no need to check.
                return true;
            }
            // Check that the confirmation is correct
            if (String.CompareOrdinal(txtSecret.Text, txtSecret2.Text) != 0) {
                MessageBox.Show(Resources.MISMATCHED_SECRETS, Resources.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                txtSecret2.Focus();
                return false;
            }
            if (!String.IsNullOrEmpty(txtUserPassword1.Text)) {
                if (String.CompareOrdinal(txtUserPassword1.Text, txtUserPassword2.Text) != 0) {
                    MessageBox.Show(Resources.MISMATCHED_PASSWORDS, Resources.ERROR, MessageBoxButtons.OK,
                                    MessageBoxIcon.Error);
                    txtUserPassword2.Focus();
                    return false;
                }
            }
            return true;
        }

        private bool SavePINsafeSettings() {
            try {
                pinsafeSettings.WriteSettingsToStorage();
                foreach (SharePointSettings.SharePointApplication app in sharePointSettings.GetLocations()) {
                    pinsafeSettings.SaveSettings(app.Location + ConfigUtil.WEB_CONFIG, app.RequiresPINsafe);
                }
                return true;
            } catch {
                return false;
            }
        }

        private bool SaveSharePointSettings() {
            try {
                if (!sharePointSettings.WriteToStorage()) {
                    return false;
                }
                if (sharePointSettings.CentralAdminExists) {
                    ConfigUtil.RegisterProvider(sharePointSettings.CentralAdminPath + ConfigUtil.WEB_CONFIG, sharePointSettings);
                }
                if (sharePointSettings.SecurityTokenExists) {
                    ConfigUtil.RegisterProvider(sharePointSettings.SecurityTokenPath + ConfigUtil.WEB_CONFIG, sharePointSettings);
                }
                foreach (SharePointSettings.SharePointApplication app in sharePointSettings.GetLocations()) {
                    ConfigUtil.RegisterFilter(app.Location + ConfigUtil.WEB_CONFIG, sharePointSettings, pinsafeSettings, app.RequiresPINsafe);
                }
                return true;
            } catch {
                return false;
            }
        }

        private void CloseClick(object sender, EventArgs e) {
            if (SaveChanges(true)) {
                Close();
            }
        }

        private static string SetPath(string initPath) {
            string startFolder = initPath ?? "";
            while (!Directory.Exists(startFolder)) {
                int slashPos = startFolder.LastIndexOf('\\');
                if (slashPos <= 0) {
                    break;
                }
                startFolder = startFolder.Substring(0, slashPos);
            }
            FolderBrowserDialog dlg = new FolderBrowserDialog { SelectedPath = startFolder };
            if (dlg.ShowDialog() == DialogResult.OK) {
                string newPath = dlg.SelectedPath;
                if (String.Compare(newPath, initPath, StringComparison.OrdinalIgnoreCase) != 0) {
                    return newPath;
                }
            }
            return initPath;
        }

        private void BtnSetCentralClick(object sender, EventArgs e) {
            lblCentralPath.Text = SetPath(lblCentralPath.Text);
        }

        private void BtnSetTokenClick(object sender, EventArgs e) {
            lblTokenPath.Text = SetPath(lblTokenPath.Text);
        }

        private void ChkCentralCheckedChanged(object sender, EventArgs e) {
            btnSetCentral.Enabled = chkCentral.Checked;
            lblCentralPath.Enabled = chkCentral.Checked;
        }

        private void ChkTokenCheckedChanged(object sender, EventArgs e) {
            btnSetToken.Enabled = chkToken.Checked;
            lblTokenPath.Enabled = chkToken.Checked;
        }

        private void MnuLoadClick(object sender, EventArgs e) {
            if (new ImportConfig().ShowDialog() == DialogResult.Cancel) {
                return;
            }
            // Reload settings
            RefreshSettings();
        }

        private void RefreshSettings() {
            sharePointSettings.ReadFromStorage();
            textSettings.LoadFromStorage();
            pinsafeSettings.ReadSettingsFromStorage();
            SettingsToDisplay();
        }

        private void BtnAddClick(object sender, EventArgs e) {
            ApplicationForm appForm = new ApplicationForm();
            if (appForm.ShowDialog(this) == DialogResult.Cancel) {
                return;
            }
            string newLocation = appForm.AppLocation;
            bool requiresPINsafe = appForm.RequiresPINsafe;
            if (lstApps.Items.Cast<SharePointSettings.SharePointApplication>().Any(app => String.Compare(newLocation, app.Location, StringComparison.OrdinalIgnoreCase) == 0)) {
                MessageBox.Show(String.Format(Resources.Web_App_Exists, newLocation), Resources.App_Exists, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }
            SharePointSettings.SharePointApplication newApp = new SharePointSettings.SharePointApplication(newLocation, requiresPINsafe);
            lstApps.Items.Add(newApp);
            CheckLocation(newApp);
        }

        private void CheckLocation(SharePointSettings.SharePointApplication app) {
            ConfigUtil.UpdateLocation(app.Location, sharePointSettings, pinsafeSettings, app.RequiresPINsafe);
            sharePointSettings.AddLocation(app.Location, app.RequiresPINsafe);
            sharePointSettings.WriteToStorage();
        }

        private void MnuShareClick(object sender, EventArgs e) {
            ConfigUtil.CreateShare();
        }

        private void BtnUpdateClick(object sender, EventArgs e) {
            if (lstApps.CheckedItems.Count == 0 && lstApps.SelectedItem != null) {
                SharePointSettings.SharePointApplication app =
                    lstApps.SelectedItem as SharePointSettings.SharePointApplication;
                if (app == null) {
                    return;
                }
                ApplicationForm appForm = new ApplicationForm(app.Location, app.RequiresPINsafe);
                if (appForm.ShowDialog(this) == DialogResult.Cancel) {
                    return;
                }
                app.RequiresPINsafe = appForm.RequiresPINsafe;
                CheckLocation(app);
                lstApps.Refresh();
                return;
            }
            foreach (SharePointSettings.SharePointApplication item in lstApps.CheckedItems) {
                ApplicationForm appForm = new ApplicationForm(item.Location, item.RequiresPINsafe);
                if (appForm.ShowDialog(this) == DialogResult.Cancel) {
                    continue;
                }
                item.RequiresPINsafe = appForm.RequiresPINsafe;
                CheckLocation(item);
                lstApps.Refresh();
            }
        }

        private void BtnRemoveClick(object sender, EventArgs e) {
            if (lstApps.CheckedItems.Count == 0) {
                MessageBox.Show(Resources.No_Items_Selected_Message, Resources.No_Items_Selected_Title,
                                MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }
            if (MessageBox.Show(Resources.Remove_PINsafe_Message, Resources.Remove_PINsafe_Title, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No) {
                return;
            }
            IList<SharePointSettings.SharePointApplication> removed = new List<SharePointSettings.SharePointApplication>(lstApps.CheckedItems.Count);
            foreach (SharePointSettings.SharePointApplication item in lstApps.CheckedItems) {
                string configFile = item.Location + ConfigUtil.WEB_CONFIG;
                ConfigUtil.UnregisterFilter(configFile);
                PINsafeSettings.RemoveSettings(configFile);
                ConfigUtil.RemoveFiles(item.Location);
                removed.Add(item);
            }
            foreach (SharePointSettings.SharePointApplication item in removed) {
                lstApps.Items.Remove(item);
                sharePointSettings.RemoveLocation(item.Location);
            }
            sharePointSettings.WriteToStorage();
        }

        private void ChkAllCheckedChanged(object sender, EventArgs e) {
            switch (chkAll.CheckState) {
            case CheckState.Checked:
                for (int idx = 0; idx < lstApps.Items.Count; idx++) {
                    lstApps.SetItemChecked(idx, true);
                }
                break;
            case CheckState.Unchecked:
                for (int idx = 0; idx < lstApps.Items.Count; idx++) {
                    lstApps.SetItemChecked(idx, false);
                }
                break;
            }
        }

        private void LstAppsItemCheck(object sender, ItemCheckEventArgs e) {
            bool allChecked = (e.NewValue == CheckState.Checked);
            bool allUnchecked = (e.NewValue == CheckState.Unchecked);
            for (int idx = 0; idx < lstApps.Items.Count; idx++) {
                if (idx == e.Index) {
                    // Ignore the one that's being changed.
                    continue;
                }
                if (lstApps.GetItemChecked(idx)) {
                    allUnchecked = false;
                } else {
                    allChecked = false;
                }
                if (!(allChecked || allUnchecked)) {
                    break;
                }
            }
            chkAll.CheckState = (allChecked
                                     ? CheckState.Checked
                                     : (allUnchecked ? CheckState.Unchecked : CheckState.Indeterminate));
        }

        private void BtnSaveClick(object sender, EventArgs e) {
            if (!SaveChanges(false)) return;
            Close();
        }

        private bool SaveChanges(bool confirm) {
            if (!CheckSecret()) {
                return false;
            }
            bool pinsafeChanged = DisplayToPINsafeSettings();
            bool sharePointChanged = DisplayToSharePointSettings() | DisplayToApplicationList();
            bool textChanged = DisplayToTextSettings();
            if (String.CompareOrdinal(sharePointSettings.ProviderName, pinsafeSettings.MembershipProvider) != 0) {
                // Membership provider name has changed - update PINsafe settings as well.
                pinsafeSettings.MembershipProvider = sharePointSettings.ProviderName;
                pinsafeChanged = true;
            }
            bool saveRequired = pinsafeChanged | sharePointChanged | textChanged;
            if (!saveRequired) {
                if (!confirm) {
                    MessageBox.Show(Resources.Save_Not_Required_Message, Resources.No_Save_Required_Title,
                                    MessageBoxButtons.OK,
                                    MessageBoxIcon.Information);
                }
                return true;
            }
            if (confirm) {
                if (MessageBox.Show(Resources.SAVE_ON_EXIT, Resources.PROGRAM_CLOSING,
                                MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No) {
                    return true;
                }
            }
            bool saveSuccessful = true;
            if (pinsafeChanged) {
                saveSuccessful &= SavePINsafeSettings();
            }
            if (sharePointChanged) {
                saveSuccessful &= SaveSharePointSettings();
            }
            if (textChanged) {
                saveSuccessful &= textSettings.SaveToStorage();
            }
            if (saveSuccessful) {
                MessageBox.Show(Resources.Config_Saved_Message, Resources.Config_Saved_Title, MessageBoxButtons.OK,
                                MessageBoxIcon.Information);
                if (MessageBox.Show(
                    Resources.MESSAGE_RESTART_IIS,
                    Resources.HEADING_RESTART_IIS, MessageBoxButtons.YesNo) == DialogResult.Yes) {
                    ConfigUtil.RestartIIS();
                }
            } else {
                MessageBox.Show(String.Format(Resources.SAVE_FAILED, Location), Resources.ERROR_SAVING,
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return true;
        }

// ReSharper disable MemberCanBeMadeStatic.Local
        private void AboutToolStripMenuItemClick(object sender, EventArgs e) {
// ReSharper restore MemberCanBeMadeStatic.Local
            new AboutDialog().Show();
        }

        private void DrpUnknownUsersSelectedIndexChanged(object sender, EventArgs e) {
            txtRedirectUrl.Enabled = (drpUnknownUsers.SelectedIndex == 2);
        }

        private void BtnAddStringClick(object sender, EventArgs e) {
            CustomStringForm form = new CustomStringForm();
            if (form.ShowDialog() == DialogResult.Cancel) {
                return;
            }
            string code = form.ErrorCode;
            string message = form.Message;
            if (string.IsNullOrEmpty(code)) {
                MessageBox.Show(Resources.No_error_code, Resources.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (lstErrors.Items.ContainsKey(code)) {
                MessageBox.Show(Resources.DUPLICATE_ERROR_CODE, Resources.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            ListViewItem item = new ListViewItem(code);
            item.SubItems.Add(message);
            lstErrors.Items.Add(item);
        }

        private void BtnEditStringClick(object sender, EventArgs e) {
            if (lstErrors.SelectedItems.Count == 0) {
                return;
            }
            ListViewItem lvItem = lstErrors.SelectedItems[0];
            string code = lvItem.Text;
            string message = lvItem.SubItems[1].Text;
            CustomStringForm form = new CustomStringForm(code, message);
            if (form.ShowDialog() == DialogResult.Cancel) {
                return;
            }
            string newMessage = form.Message;
            if (string.CompareOrdinal(message, newMessage) == 0) {
                return;
            }
            lvItem.SubItems[1].Text = newMessage;
            Refresh();
        }

        private void BtnDeleteStringClick(object sender, EventArgs e) {
            if (lstErrors.SelectedItems.Count == 0) {
                return;
            }
            foreach (ListViewItem selectedItem in lstErrors.SelectedItems) {
                lstErrors.Items.Remove(selectedItem);
            }
        }

		private void PinResetCheckedChanged(object sender, EventArgs e) {
			if (chkPinReset.Checked) {
				chkPwdReset.Enabled = true;
				txtResetDisabled.Enabled = false;
			} else {
				// You can't have password reset without PIN reset.
				chkPwdReset.Checked = false;
				chkPwdReset.Enabled = false;
				txtResetDisabled.Enabled = true;
			}
		}

		private void ChkManagePasswordsCheckedChanged(object sender, EventArgs e) {
			bool enabled = chkManagePasswords.Checked;
			drpUsernameType.Enabled = enabled;
			txtUserAccount.Enabled = enabled;
			txtUserPassword1.Enabled = enabled;
			txtUserPassword2.Enabled = enabled;
		}
    }
}
