﻿using System;
using System.Windows.Forms;
using com.swivelsecure.pinsafe.config.Properties;

namespace com.swivelsecure.pinsafe.config {
    public partial class ImportConfig : Form {
        public ImportConfig() {
            InitializeComponent();
            SharePointSettings settings = new SharePointSettings();
            txtServer.Text = settings.ConfigServer;
        }

        private void BtnCancelClick(object sender, EventArgs e) {
            MessageBox.Show(Resources.Import_Info, Resources.Import_Cancelled, MessageBoxButtons.OK, MessageBoxIcon.Information);
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void BtnImportClick(object sender, EventArgs e) {
            string configServer = txtServer.Text;
            if (String.IsNullOrEmpty(configServer)) {
                MessageBox.Show(Resources.No_Server_Message, Resources.No_Server_Title, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (!PINsafeSettings.SharedConfigFileExists(configServer, "")) {
                MessageBox.Show(String.Format(Resources.Server_Not_Found_Message, configServer), Resources.Server_Not_Found_Title, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            SharePointSettings sharePointSettings = new SharePointSettings {ConfigServer = configServer};
            if (!sharePointSettings.ReadFromSharedConfig()) {
                MessageBox.Show(String.Format(Resources.Server_Read_Error_Message, configServer), Resources.Server_Read_Error_Title, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            // We've probably lost the configuration server now.
            sharePointSettings.ConfigServer = configServer;
            PINsafeSettings pinsafeSettings = PINsafeSettings.SettingsFromShared(configServer);
            if (pinsafeSettings == null) {
                MessageBox.Show(String.Format(Resources.Server_Read_Error_Message, configServer), Resources.Server_Read_Error_Title, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            TextSettings textSettings = new TextSettings();
            if (!textSettings.LoadFromCentralConfig(configServer)) {
                MessageBox.Show(String.Format(Resources.Server_Read_Error_Message, configServer), Resources.Server_Read_Error_Title, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            ConfigUtil.StopIIS();
            sharePointSettings.WriteToStorage();
            textSettings.SaveToStorage();
            pinsafeSettings.WriteSettingsToStorage();
            if (sharePointSettings.CentralAdminExists) {
                ConfigUtil.RegisterProvider(sharePointSettings.CentralAdminPath + ConfigUtil.WEB_CONFIG, sharePointSettings);
            }
            if (sharePointSettings.SecurityTokenExists) {
                ConfigUtil.RegisterProvider(sharePointSettings.SecurityTokenPath + ConfigUtil.WEB_CONFIG, sharePointSettings);
            }
            foreach (SharePointSettings.SharePointApplication app in sharePointSettings.GetLocations()) {
               ConfigUtil.UpdateLocation(app.Location, sharePointSettings, pinsafeSettings, app.RequiresPINsafe);
            }
            ConfigUtil.StartIIS();
            MessageBox.Show(String.Format(Resources.Settings_Imported_Message, configServer), Resources.Settings_Imported_Title, MessageBoxButtons.OK, MessageBoxIcon.Information);
            DialogResult = DialogResult.OK;
            Close();
        }
    }
}
