﻿using System;
using System.IO;
using System.Windows.Forms;
using com.swivelsecure.pinsafe.config.Properties;

namespace com.swivelsecure.pinsafe.config
{
    static class Program {
        private const string DLL_PATH = "web\\bin";
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static int Main(string[] args) {
            try {
                Application.EnableVisualStyles();
                Application.SetCompatibleTextRenderingDefault(false);
                bool install = false;
                bool uninstall = false;
                if (args != null && args.Length > 0 && args[0].Length > 0) {
                    char cmd = args[0][0];
                    if (cmd == '-' && args[0].Length > 1) {
                        cmd = args[0][1];
                    }
                    cmd = Char.ToLower(cmd);
                    install = (cmd == 'i');
                    uninstall = (cmd == 'u');
                    if (!install && !PINsafeSettings.ConfigFileExists("")) {
                        install = true; // You must run the configuration program at least once.
                    }
                }
                string dllLoc = Application.ExecutablePath;
                int idx = dllLoc.LastIndexOf('\\');
                dllLoc = ((idx < 0) ? "" : dllLoc.Substring(0, idx + 1)) + DLL_PATH;
                if (uninstall) {
                    // Stop IIS.
                    ConfigUtil.StopIIS();
                    SharePointSettings spSettings = new SharePointSettings();
                    // Undeploy all installations.
                    foreach (SharePointSettings.SharePointApplication app in spSettings.GetLocations()) {
                        string location = app.Location;
                        string configLocation = location + ConfigUtil.WEB_CONFIG;
                        try {
                            PINsafeSettings.RemoveSettings(configLocation);
                            ConfigUtil.UnregisterFilter(configLocation);
                            ConfigUtil.RemoveFiles(location);
                        } catch {
                            // File is not valid. Never mind.
                        }
                    }
                    ConfigUtil.UninstallGac(dllLoc);
                    // Remove the network share, if any.
                    ConfigUtil.RemoveShare();
                    ConfigUtil.StartIIS();
                } else {
					// Check if the password settings file exists.
					if (!PINsafeSettings.ConfigFileExists(PasswordGenerator.SETTINGS_FILE)) {
						string fileLoc = Application.ExecutablePath;
						int slashPos = fileLoc.LastIndexOf('\\');
						fileLoc = ((slashPos < 0) ? "" : fileLoc.Substring(0, slashPos + 1)) + PasswordGenerator.SETTINGS_FILE;
						if (File.Exists(fileLoc)) {
							// If it doesn't exist, what then?
							File.Move(fileLoc, PINsafeSettings.CommonDataPath + PasswordGenerator.SETTINGS_FILE);
						}
					}
                    ConfigForm configForm = new ConfigForm();
                    if (install) {
                        ConfigUtil.InstallGac(dllLoc);
                        Startup startup = new Startup();
                        if (startup.ShowDialog() == DialogResult.Cancel) {
                            return 0;
                        }
                        switch (startup.InstallationType) {
                            case Startup.InstallType.New:
                                MessageBox.Show(Resources.Share_Config_Message, Resources.New_Installation,
                                                MessageBoxButtons.OK, MessageBoxIcon.Information);
                                break;
                            case Startup.InstallType.Copy:
                                new ImportConfig().ShowDialog();
                                // Reload the configuration
                                configForm = new ConfigForm();
                                break;
                            case Startup.InstallType.UpgradeNew:
                                ConfigUtil.Upgrade(dllLoc);
                                configForm = new ConfigForm();
                                break;
                            case Startup.InstallType.UpgradeOld:
                                new UpgradeSettings().ShowDialog();
                                // Reload the configuration
                                configForm = new ConfigForm();
                                break;
                        }
                    }
                    Application.Run(configForm);
                }
                return 1;
            } catch (Exception e) {
                MessageBox.Show(String.Format(Resources.Display_Error, e.Message, e.StackTrace),
                                Resources.Filter_Config_Failed, MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
                return 0;
            }
        }
    }        
}
