﻿using System;
using System.IdentityModel.Tokens;
using System.Text;
using System.Web;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.IdentityModel;
using com.swivelsecure.pinsafe;

namespace Web._forms {
    public partial class ChangePW : System.Web.UI.Page {

	    private const string LOG_SOURCE = "CHANGE_PASSWORD";

        private TextSettings text;
        private PINsafeSettings settings;
	    private DebugLogger logger;

	    public ChangePW() {
	        Load += PageLoad;
        }

	    protected void PageLoad(object sender, EventArgs e) {
            text = new TextSettings();
            settings = PINsafeSettings.SettingsFromConfiguration;
			logger = settings.Logger;

            // Set the labels.
            lblUsername.Text = text.Username + ":";
            lblOldPassword.Text = text.OldPassword + ":";
            lblNewPassword1.Text = text.NewPassword1 + ":";
            lblNewPassword2.Text = text.NewPassword2 + ":";
            imgLogo.Src = text.LogoFile;
            btnChangePassword.Text = text.ChangePasswordButton;
            Title = text.ChangePasswordPageTitle;
            lblTitle.Text = text.ChangePasswordPageHeader;
            lblFooter.Text = text.ChangePasswordPageFooter;
			if (settings.ResetPinEnabled && settings.ResetPasswordEnabled) {
				resetRow.Visible = true;
				lnkReset.Text = text.ForgottenPassword;
			} else {
				resetRow.Visible = false;
			}
			if (!settings.ManageADPasswords) {
				txtUsername.Enabled = false;
				txtOldPassword.Enabled = false;
				txtNewPassword1.Enabled = false;
				txtNewPassword2.Enabled = false;
				btnChangePassword.Enabled = false;
				SetError(text.ChangePasswordDisabled);
				return;
			}
		    
            if (IsPostBack)
                return;
            lblResult.Text = "";
            string username = null;
            HttpCookie cpCookie = Request.Cookies[Login.CPWD_COOKIE_NAME];
            if (cpCookie != null) {
                username = cpCookie.Value;
            } else {
                cpCookie = Request.Cookies[Login.COOKIE_NAME];
                PINsafeClient client = new PINsafeClient(settings, text);
                if (cpCookie == null) {
                    cpCookie = Request.Cookies[Login.CP_COOKIE_NAME];
                    if (cpCookie != null) {
                        username = client.GetCPCookieUsername(cpCookie.Value);
                    }
                } else {
                    username = client.GetCookieUsername(cpCookie.Value);
                }
            }
            if (username == null) {
                username = Request["username"];
            }
            if (!String.IsNullOrEmpty(username)) {
                txtUsername.Text = username;
                txtUsername.ReadOnly = true;
				if (resetRow.Visible) {
					lnkReset.NavigateUrl = "./Reset.aspx?username=" + username;
				}
			}
            if (!String.IsNullOrEmpty(Request["Source"])) {
                hfRedirectUrl.Value = Request["Source"];
            }
            string changeRequired = Request["changerequired"];
            if (changeRequired != null) {
                SetInfo(string.Compare(changeRequired, "both", StringComparison.OrdinalIgnoreCase) == 0
                            ? "You must change your password and PIN before next login."
                            : "You must change your password before next login.");
            }
        }

        private void SetError(string message) {
            SetComment(message, Login.CSS_ERROR_CLASS);
        }

        private void SetInfo(string message) {
            SetComment(message, Login.CSS_INFO_CLASS);
        }

        private void SetComment(string message, string cssClass) {
            lblResult.Text = message;
            lblResult.CssClass = cssClass;
        }

        protected void BtnChangePasswordClick(object sender, EventArgs e) {
            // Sanity check
			if (!settings.ManageADPasswords) {
				SetError(text.ChangePasswordDisabled);
				return;
			}
            string username = txtUsername.Text;
            if (String.IsNullOrEmpty(username)) {
                SetError("No username entered");
                return;
            }
            string oldPassword = txtOldPassword.Text;
            if (String.IsNullOrEmpty(oldPassword)) {
                SetError("Old Password not entered");
                return;
            }
            string newPassword = txtNewPassword1.Text;
            if (String.IsNullOrEmpty(newPassword)) {
                SetError("New Password not entered");
                return;
            }
            if (!newPassword.Equals(txtNewPassword2.Text)) {
                SetError("New Passwords do not match");
                return;
            }
            ActiveDirectoryClient client = new ActiveDirectoryClient(settings, text);
	        if (client.ChangePassword(username, oldPassword, newPassword)) {
				logger.WriteLine(LOG_SOURCE, "Password successfully changed for \"{0}\"", username);
                StringBuilder linkText = new StringBuilder("Password successfully changed<br/>");
                bool loggedOn = false;
                bool pinChangeRequired = false;
                HttpCookie cookie = Request.Cookies[Login.CP_COOKIE_NAME];
                PINsafeClient psClient = new PINsafeClient(settings, text);
                if (cookie != null) {
                    if (psClient.CPCookieValid(cookie.Value)) {
//						logger.WriteLine(LOG_SOURCE, "PIN change required for user \"{0}\"", username);
                        loggedOn = true;
                        pinChangeRequired = true;
                    }
                }
                if (!pinChangeRequired) {
                    cookie = Request.Cookies[Login.COOKIE_NAME];
                    if (cookie != null) {
                        bool? pinsafeRequired = settings.RequiresPINsafe;
                        if ((pinsafeRequired == null || (bool) pinsafeRequired)
                                               ? psClient.TwoFactorCookieValid(cookie.Value)
                                               : psClient.SingleFactorCookieValid(cookie.Value)) {
//							logger.WriteLine(LOG_SOURCE, "Authentication complete for user \"{0}\"", username);
                            loggedOn = true;
                        }
                    }
                }
                if (loggedOn) {
                    // Already logged on to PINsafe: try AD
                    bool spLoggedOn = CheckSharePointAuthentication(username, newPassword, linkText);
//                    logger.WriteLine(LOG_SOURCE, spLoggedOn ? "SharePoint authentication successful for user \"{0}\"" : "SharePoint authentication failed for user \"{0}\"", username);

                    string source;

                    // We don't need the password change cookie any more.
                    HttpCookie cpwdCookie = new HttpCookie(Login.CPWD_COOKIE_NAME, "") { Expires = DateTime.MinValue };
                    Response.SetCookie(cpwdCookie);
                    if (pinChangeRequired) {
                        /*
                                                linkText.Append("You must change your PIN before connecting to SharePoint.<br/>");
                                                linkText.Append("Click <a href='{0}'>here</a> to change your PIN");
                        */
                        source = hfRedirectUrl.Value;
                        if (String.IsNullOrEmpty(source)) {
                            source = Request["Source"];
                        }
                        string cpUrl = settings.ChangePinUrl + "?changerequired=pin";
                        if (!String.IsNullOrEmpty(source)) {
                            cpUrl += "&source=" + source;
                        }
//                        logger.WriteLine(LOG_SOURCE, "Redirecting user \"{0}\" to change PIN page", username);
                        Response.Redirect(cpUrl, true);
                        return;
                    }
                    if (spLoggedOn) {
                        linkText.Append("Click <a href='{0}'>here</a> to continue to SharePoint");
                        source = hfRedirectUrl.Value;
                        if (String.IsNullOrEmpty(source)) {
                            source = Request["Source"];
                        }
                    } else {
                        linkText.Append("Click <a href='{0}'>here</a> to log on");
                        source = settings.LogonUrl + "?Username=" + username;
                    }
                    SetInfo(string.Format(linkText.ToString(), source));
                }
            } else {
                string error = client.Error;
                if (string.IsNullOrEmpty(error)) {
                    error = "Password change failed";
                }
				logger.WriteLine(LOG_SOURCE, "Change Password failed for user \"{0}\": {1}", username, error);
                SetError(error);
            }
        }

        private bool CheckSharePointAuthentication(string username, string password, StringBuilder linkText) {
            SPContext context = SPContext.Current;
            if (context == null) {
                linkText.Append("Sharepoint context not found<br/>");
//				logger.WriteLine(LOG_SOURCE, "Sharepoint context not found");
				return false;
            }
            StringBuilder url = new StringBuilder(Request.Url.Scheme);
            url.Append("://");
            url.Append(Request.Url.Host);
            if (!Request.Url.IsDefaultPort) {
                url.Append(":");
                url.Append(Request.Url.Port);
            }
            Uri currentUri = null;
            SPWebApplication webapp;
            try {
                currentUri = new Uri(url.ToString());
                webapp = SPWebApplication.Lookup(currentUri);
            } catch {
                // Ignore.
                webapp = null;
            }
            if (webapp == null) {
                if (String.IsNullOrEmpty(settings.WebApplication)) {
                    linkText.Append("Sharepoint web application \"" + url + "\" not found");
					logger.WriteLine(LOG_SOURCE, "Sharepoint web application \"{0}\" not found", url);
					return false;
                }
                currentUri = new Uri(settings.WebApplication);
                webapp = SPWebApplication.Lookup(currentUri);
                if (webapp == null) {
                    linkText.Append("Sharepoint web application not found for \"" + url + "\" or for \"" +
                             settings.WebApplication + "\"");
					logger.WriteLine(LOG_SOURCE, "Sharepoint web application not found for \"{0}\" or for \"{1}\"", url, settings.WebApplication);
					return false;
                }
            }
            SPIisSettings iisSettings = webapp.IisSettings[SPUrlZone.Default];
            if (iisSettings == null) {
                linkText.Append("IIS settings not found");
                return false;
            }
            SPFormsAuthenticationProvider authProvider = iisSettings.FormsClaimsAuthenticationProvider;
            if (authProvider == null) {
                linkText.Append("Authentication provider not found");
                return false;
            }
            SecurityToken token = SPSecurityContext.SecurityTokenForFormsAuthentication(currentUri,
                                                                                             authProvider.MembershipProvider,
                                                                                             authProvider.RoleProvider,
                                                                                             username, password);
            if (token == null) {
                //linkText.Append(text.SharePointLoginError);
                //linkText.Append("<br/>");
//				logger.WriteLine(LOG_SOURCE, "SharePoint authentication failed for user \"{0}\"", username);
                return false;
            }
            SPFederationAuthenticationModule fedmod = SPFederationAuthenticationModule.Current;
            if (fedmod == null) {
                linkText.Append("Federation authentication module not found");
				logger.WriteLine(LOG_SOURCE, "Federation authentication module not found");
				return false;
            }
            fedmod.SetPrincipalAndWriteSessionToken(token);
            return true;
        }
    }
}