﻿using System;
using System.Web;
using System.Web.UI;
using com.swivelsecure.pinsafe;

namespace Web._forms {
    public partial class ChangePin : Page {
        private TextSettings text;
        private PINsafeSettings settings;

        public ChangePin() {
            Load += PageLoad;
        }

        protected void PageLoad(object sender, EventArgs e) {
            text = new TextSettings();
            settings = PINsafeSettings.SettingsFromConfiguration;

            // Set the labels.
            lblUsername.Text = text.Username + ":";
            lblOldOtc.Text = text.OldOtc + ":";
            lblNewOtc1.Text = text.NewOtc1 + ":";
            lblNewOtc2.Text = text.NewOtc2 + ":";
            imgLogo.Src = text.LogoFile;
            btnChangePin.Text = text.ChangePinButton;
            btnImage.Value = text.ShowImageButton;
            Title = text.ChangePinPageTitle;
            lblTitle.Text = text.ChangePinPageHeader;
            lblFooter.Text = text.ChangePinPageFooter;
            if (IsPostBack) return;
            lblResult.Text = "";
            imgRow.Visible = settings.ShowTURing;
	        resetRow.Visible = settings.ResetPinEnabled;
			if (resetRow.Visible) {
				lnkReset.Text = text.ForgottenPin;
			}
            HttpCookie cpCookie = Request.Cookies[Login.CP_COOKIE_NAME];
            PINsafeClient client = new PINsafeClient(settings, text);
            string username = (cpCookie != null) ? client.GetCPCookieUsername(cpCookie.Value) : Request["username"];
            if (!String.IsNullOrEmpty(username)) {
                txtUsername.Text = username;
                txtUsername.ReadOnly = true;
                if (!IsPostBack) {
                    imgTuring.Src = "./SCImage.aspx?username=" + username + "&random=" + new Random().Next();
                }
				if (resetRow.Visible) {
					lnkReset.NavigateUrl = "./Reset.aspx?username=" + username;
				}
            }
            if (!String.IsNullOrEmpty(Request["Source"])) {
                hfRedirectUrl.Value = Request["Source"];
            }
            if (!string.IsNullOrEmpty(Request["changeRequired"])) {
                SetInfo("You must change your PIN before next login.");
            }
        }

        private void SetError(string message) {
            SetComment(message, Login.CSS_ERROR_CLASS);
        }

        private void SetInfo(string message) {
            SetComment(message, Login.CSS_INFO_CLASS);
        }

        private void SetComment(string message, string cssClass) {
            lblResult.Text = message;
            lblResult.CssClass = cssClass;
        }

        protected void BtnChangePinClick(object sender, EventArgs e) {
            // Sanity check
            string username = txtUsername.Text;
            if (String.IsNullOrEmpty(username)) {
                SetError("No username entered");
                return;
            }
            string oldOtc = txtOldOtc.Text;
            if (String.IsNullOrEmpty(oldOtc)) {
                SetError("Old OTC not entered");
                return;
            }
            string newOtc = txtNewOtc1.Text;
            if (String.IsNullOrEmpty(newOtc)) {
                SetError("New OTC not entered");
                return;
            }
            if (!newOtc.Equals(txtNewOtc2.Text)) {
                SetError("New OTCs do not match");
                return;
            }
            PINsafeClient client = new PINsafeClient(settings, text);
            if (!client.ChangePin(username, "", "", oldOtc, newOtc)) {
                imgRow.Visible = false;
                string error = client.ErrorMessage;
                if (String.IsNullOrEmpty(error)) {
                    SetError(text.ChangePinError);
                } else {
                    SetError(text.ChangePinError + ":<br/>" + error);
                }
            } else {
                imgRow.Visible = false;
                string source = Request["Source"];
                DateTime expiryTime = DateTime.Now.AddHours(2.0);
                HttpCookie cpCookie = Request.Cookies[Login.CP_COOKIE_NAME];
                if (cpCookie != null && client.CPCookieValid(cpCookie.Value)) {
                    string cookieUsername = client.GetCPCookieUsername(cpCookie.Value);
                    if (String.Compare(cookieUsername, username, StringComparison.OrdinalIgnoreCase) != 0) {
                        // Wrong username.
                        SetError("Wrong username for Change PIN");
                        return;
                    }
                    string pinsafeCookie = client.GetPINsafeCookie(username, expiryTime);
                    if (pinsafeCookie != null) {
                        // Set the PINsafe cookie
                        HttpCookie cookie = new HttpCookie(Login.COOKIE_NAME, pinsafeCookie) { Expires = expiryTime, HttpOnly = true };
                        if (!String.IsNullOrEmpty(settings.AuthenticationDomain)) {
                            cookie.Domain = settings.AuthenticationDomain;
                        }
                        Response.SetCookie(cookie);
                        // Remove the CP required cookie.
                        cpCookie = new HttpCookie(Login.CP_COOKIE_NAME) { Value = "", Expires = DateTime.MinValue };
                        Response.SetCookie(cpCookie);
                    }
                }
                if (String.IsNullOrEmpty(source)) {
                    source = hfRedirectUrl.Value;
                }
                HttpCookie authCookie = Request.Cookies[Login.FEDAUTH_COOKIE_NAME];
                if (authCookie != null) {
                    string authUrl = settings.AuthUrl;
                    if (!string.IsNullOrEmpty(authUrl)) {
                        if (!string.IsNullOrEmpty(source)) {
                            authUrl += (authUrl.IndexOf('?') > 0) ? "&" : "?";
                            authUrl += "Source=" + source;
                        }
                        SetInfo(String.Format("Change PIN succeeded, click <a href=\"{0}\">here</a> to continue", authUrl));
                    } else if (!string.IsNullOrEmpty(source)) {
                        SetInfo(String.Format("Change PIN succeeded, click <a href=\"{0}\">here</a> to continue", source));
                    } else {
                        SetInfo("Change PIN succeeded");
                    }
                } else {
                    string authUrl = settings.LogonUrl;
                    if (!string.IsNullOrEmpty(source)) {
                        authUrl += (authUrl.IndexOf('?') > 0) ? "&" : "?";
                        authUrl += "Source=" + source;
                    }
                    SetInfo(String.Format("Change PIN succeeded, click <a href=\"{0}\">here</a> to log on", authUrl));
                }
            }
        }
    }
}