﻿using System;
using System.IdentityModel.Tokens;
using System.Text;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using com.swivelsecure.pinsafe;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.IdentityModel;

namespace Web._forms {
    public partial class Login : Page {
        public const string COOKIE_NAME = "PINSAFE";
        public const string CP_COOKIE_NAME = "PINSAFECP";
        public const string CPWD_COOKIE_NAME = "CHANGEPWD";
        public const string FEDAUTH_COOKIE_NAME = "FedAuth";
        public const string CSS_ERROR_CLASS = "error";
        public const string CSS_INFO_CLASS = "info";
        private const string WARN_PIN_EXPIRY = "AGENT_WARN_PIN_EXPIRY";
        private const string ERROR_PIN_EXPIRED = "AGENT_WARN_CHANGE_PIN";
        private const string LOG_SOURCE = "LOGIN";
        private string membershipProvider = "";
        private PINsafeSettings settings;
        private TextSettings text;
        private bool loggedIn;

        public Login() {
            loggedIn = false;
            Load += PageLoad;
        }

        protected void PageLoad(object sender, EventArgs e) {
            settings = PINsafeSettings.SettingsFromConfiguration;
            text = new TextSettings();

            // Set the labels.
            lblUsername.Text = text.Username + ":";
            lblPassword.Text = text.Password + ":";
            lblOTC.Text = text.Otc + ":";
            imgLogo.Src = text.LogoFile;
            btnLogin.Text = text.LoginButton;
            btnLogin2.Text = text.LoginButton;
            btnImage.Value = text.ShowImageButton;
            Title = text.PageTitle;
            lblTitle.Text = text.PageHeader;
            lblFooter.Text = text.PageFooter;
            if (!IsPostBack) {
                lblResult.Text = "";
                if (!String.IsNullOrEmpty(Request["Source"])) {
                    hfRedirectUrl.Value = Request["Source"];
                }
                if (!String.IsNullOrEmpty(Request["Username"])) {
                    txtUsername.Text = Request["Username"];
                }
            }

            membershipProvider = settings.MembershipProvider;
            if (String.IsNullOrEmpty(membershipProvider)) {
                MembershipProvider p = Membership.Provider;
                if (p != null) {
                    membershipProvider = p.Name;
                }
            }
            imgRow.Visible = ((settings.ShowTURing || settings.ShowMessage) && !loggedIn);
            ClientScriptManager csm = Page.ClientScript;
            bool? requiresPINsafe = settings.RequiresPINsafe;
            if (requiresPINsafe == null || (bool)requiresPINsafe) {
                if (settings.AllowNonPINsafeUsers) {
                    btnLogin2.Style[HtmlTextWriterStyle.Display] = "block";
                    otcRow.Style[HtmlTextWriterStyle.Display] = "none";
                    imgRow.Style[HtmlTextWriterStyle.Display] = "none";
                    if (String.IsNullOrEmpty(settings.RedirectUrl)) {
                        csm.RegisterClientScriptInclude("pinsafeScript", "./pinsafe2.js?mobile=0");
                    } else {
                        csm.RegisterClientScriptInclude("pinsafeScript", "./pinsafe3.js?mobile=0");
                        hfRedirectUrl.Value = settings.RedirectUrl;
                    }
                } else {
                    btnLogin2.Style[HtmlTextWriterStyle.Display] = "none";
                    otcRow.Style[HtmlTextWriterStyle.Display] = "";
                    imgRow.Style[HtmlTextWriterStyle.Display] = "";
                    csm.RegisterClientScriptInclude("pinsafeScript", "./pinsafe1.js?mobile=0");
                }
				resetRow.Visible = settings.ResetPinEnabled;
				lnkReset.Text = text.ForgottenPinOrPassword;
			} else {
                // No PINsafe required.
                btnLogin2.Style[HtmlTextWriterStyle.Display] = "block";
                otcRow.Style[HtmlTextWriterStyle.Display] = "none";
                imgRow.Style[HtmlTextWriterStyle.Display] = "none";
                csm.RegisterClientScriptInclude("pinsafeScript", "./pinsafe4.js?mobile=0");
				resetRow.Visible = false;
			}
        }

        protected void BtnLoginClick(object sender, EventArgs e) {
            // Try to authenticate.
            string username = txtUsername.Text;
            string password = txtPassword.Text;
            string otc = txtOTC.Text;

            if (String.IsNullOrEmpty(username)) {
                SetError(text.UsernameMissing);
                return;
            }
            if (String.IsNullOrEmpty(password)) {
                SetError(text.PasswordMissing);
                return;
            }
            PINsafeClient client = new PINsafeClient(settings, text);
            bool loginRequired = LoginRequired(username, client);
            if (loginRequired && String.IsNullOrEmpty(otc)) {
                SetError(text.OtcMissing);
                return;
            }
            DebugLogger logger = settings.Logger;
            string source = RedirectSource;
	        ActiveDirectoryClient adclient = new ActiveDirectoryClient(settings, text);
		    int authenticateRetval = adclient.CheckPassword(username, password);
			bool passwordChangeRequired = (authenticateRetval == ActiveDirectoryClient.ERR_PWD_EXPIRED || authenticateRetval == ActiveDirectoryClient.ERR_PWD_CHANGE_REQ);
		    bool adAuthenticated = (authenticateRetval == 0) || passwordChangeRequired;
		    if (passwordChangeRequired) {
			    logger.WriteLine(LOG_SOURCE, "User \"{0}\" must change their password.", username);
		    }
	        DateTime expiryTime = DateTime.Now.AddHours(2.0);
            string pinsafeCookie = GetPinsafeCookie(username, otc, expiryTime, loginRequired, adAuthenticated || (authenticateRetval == ActiveDirectoryClient.ERR_AD_DISABLED), client);
            if (pinsafeCookie != null) {
//                logger.WriteLine(LOG_SOURCE, "PINsafe authentication for user \"{0}\" successful", username);
                string[] warnings = client.Warnings;
                bool pinChangeRequired = false;
                bool pinExpired = false;
                if (warnings != null && warnings.Length > 0 && warnings[0] != null) {
                    if (warnings[0].Equals(WARN_PIN_EXPIRY)) {
//                        logger.WriteLine(LOG_SOURCE, "PIN will expire soon for user \"{0}\"", username);
                        pinChangeRequired = true;
                        SetChangePINCookie(expiryTime, client, username);
                    }
                    if (warnings[0].Equals(ERROR_PIN_EXPIRED)) {
//                        logger.WriteLine(LOG_SOURCE, "PIN has expired for user \"{0}\"", username);
                        // PIN expired - cannot log in.
                        pinExpired = true;
                    }
                }
                if (pinExpired) {
//                    logger.WriteLine(LOG_SOURCE, "PIN expired for user \"{0}\"", username);
                    // PIN expired - cannot log in.
                    SetError(text.GetText(ERROR_PIN_EXPIRED));
                    imgRow.Visible = false;
                    return;
                }
                if (!passwordChangeRequired) {
                    if (pinChangeRequired) {
//                        logger.WriteLine(LOG_SOURCE, "Redirecting user \"{0}\" to PIN change page,", username);
                        StringBuilder cpUrl = new StringBuilder(settings.ChangePinUrl);
                        cpUrl.Append((settings.ChangePinUrl.IndexOf('?') > 0) ? "&" : "?");
                        cpUrl.Append("username=");
                        cpUrl.Append(username);
                        cpUrl.Append("&changerequired=pin");
                        if (!String.IsNullOrEmpty(source)) {
                            cpUrl.Append("&Source=");
                            cpUrl.Append(source);
                        }
                        Response.Redirect(cpUrl.ToString(), true);
                        return;
                    }
                    SecurityToken token = SPAuthenticate(username, password);
//                    logger.WriteLine(LOG_SOURCE, "SharePoint authentication for user \"{0}\" {1}", username, (token == null) ? "failed" : "succeeded");
                    bool spAuthenticated = (token != null);
                    if (spAuthenticated) {
//                        logger.WriteLine(LOG_SOURCE, "Attempting to authenticate user \"{0}\" to SharePoint", username);
                        if (!LoginSharePoint(token)) {
//                            logger.WriteLine(LOG_SOURCE, "Unable to authenticate user \"{0}\" to SharePoint.", username);
                            SetError(text.SharePointLoginError);
                            return;
                        }
//                        logger.WriteLine(LOG_SOURCE, "Setting PINsafe authentication cookie for \"{0}\"", username);
                        // Set the PINsafe cookie
                        HttpCookie cookie = GetPINsafeCookie(expiryTime, pinsafeCookie);
                        Response.SetCookie(cookie);
                        string authUrl = settings.AuthUrl;
                        if (!string.IsNullOrEmpty(authUrl)) {
                            if (!string.IsNullOrEmpty(source)) {
                                authUrl += "?Source=" + source;
                                authUrl += "&mobile=0";
                            } else {
                                authUrl += "?mobile=0";
                            }
//                            logger.WriteLine(LOG_SOURCE, "Redirecting user \"{0}\" to authentication URL \"{1}\"", username, authUrl);
                            Response.Redirect(authUrl);
                        } else if (!string.IsNullOrEmpty(source)) {
//                            logger.WriteLine(LOG_SOURCE, "Redirecting user \"{0}\" to source URL \"{1}\"", username, authUrl);
                            Response.Redirect(source);
                        } else {
//                            logger.WriteLine(LOG_SOURCE, "No redirection specified for user \"{0}\"", username);
                            SetInfo("Authentication succeeded");
                        }
//                    } else {
//                        logger.WriteLine(LOG_SOURCE, "SharePoint token for \"{0}\" is null", username);
                    }
                }
                if (passwordChangeRequired) {
                    logger.WriteLine(LOG_SOURCE, "Authentication successful for user \"{0}\", but password change required.", username);
                    String cpUrl = ChangePasswordUrl(source, username);
                    HttpCookie cpwdCookie = new HttpCookie(CPWD_COOKIE_NAME, username);
                    Response.SetCookie(cpwdCookie);
                    if (pinChangeRequired) {
                        cpUrl += "&changerequired=both";

                    } else {
                        cpUrl += "&changerequired=password";
                    }
                    imgRow.Visible = false;
                    if (!pinChangeRequired) {
                        // Set the PINsafe cookie
                        HttpCookie cookie = GetPINsafeCookie(expiryTime, pinsafeCookie);
                        Response.SetCookie(cookie);
                    }
                    Response.Redirect(cpUrl, true);
                } else if (!loggedIn) {
                    logger.WriteLine(LOG_SOURCE, "Authentication failed for user \"{0}\"", username);
                    SetError(text.SharePointLoginError);
                }
//            } else if (spAuthenticated) {
//                if (adAuthenticated) {
//                    logger.WriteLine(LOG_SOURCE, "User \"{0}\" was authenticated successfully by AD (return code = {1}), but could not authenticate to PINsafe.", username, authenticateRetval);
//                    SetError(text.PINsafeLoginError);
//                } else {
//                    logger.WriteLine(LOG_SOURCE, "User \"{0}\" failed to authenticate to AD despite SharePoint authentication succeeding. AD return code was {1}", username, authenticateRetval);
//                    SetError(text.SharePointLoginError + ": LoginUser returned " + authenticateRetval);
//                }
            } else if (authenticateRetval == 0) {
//                logger.WriteLine(LOG_SOURCE, "User \"{0}\" authenticated successfully to AD, but was rejected by SharePoint", username);
                SetError(text.SharePointLoginError /*+ " (LoginUser succeeded)"*/);
            } else {
//                logger.WriteLine(LOG_SOURCE, "User \"{0}\" failed to authenticate to AD: return code was {1}", username, authenticateRetval);
                SetError(text.SharePointLoginError /*+ ": LoginUser returned " + authenticateRetval*/);
            }
//            logger.WriteLine(LOG_SOURCE, "End of authentication process for user \"{0}\"", username);
        }

        private static string GetPinsafeCookie(string username, string otc, DateTime expiryTime, bool loginRequired, bool authenticated,
                                               PINsafeClient client) {
            string pinsafeCookie = authenticated ? (loginRequired
                                       ? client.Login(username, "", otc, expiryTime)
                                       : client.GetSingleFactorCookie(username, expiryTime))
                                       : null;
            return pinsafeCookie;
        }

        private string RedirectSource {
            get {
                string source = hfRedirectUrl.Value;
                if (String.IsNullOrEmpty(source)) {
                    source = Request["Source"];
                }
                return source;
            }
        }

        private void SetChangePINCookie(DateTime expiryTime, PINsafeClient client, string username) {
            string pinsafeCookie = client.GetChangePINCookie(username, expiryTime);
            HttpCookie cookie = new HttpCookie(CP_COOKIE_NAME, pinsafeCookie) { Expires = expiryTime, HttpOnly = true };
            if (!String.IsNullOrEmpty(settings.AuthenticationDomain)) {
                cookie.Domain = settings.AuthenticationDomain;
            }
            Response.SetCookie(cookie);
        }

        private HttpCookie GetPINsafeCookie(DateTime expiryTime, string pinsafeCookie) {
            HttpCookie cookie = new HttpCookie(COOKIE_NAME, pinsafeCookie) { Expires = expiryTime, HttpOnly = true };
            //cookie.Secure=true;
            if (!String.IsNullOrEmpty(settings.AuthenticationDomain)) {
                cookie.Domain = settings.AuthenticationDomain;
            }
            return cookie;
        }

        private String ChangePasswordUrl(string source, string username) {
            StringBuilder cpUrl = new StringBuilder(settings.ChangePasswordUrl);
            cpUrl.Append((settings.ChangePinUrl.IndexOf('?') > 0) ? "&" : "?");
            cpUrl.Append("username=");
            cpUrl.Append(username);
            if (!String.IsNullOrEmpty(source)) {
                cpUrl.Append("&Source=");
                cpUrl.Append(source);
            }
            return cpUrl.ToString();
        }

        /*
        private bool CheckPasswordChangeRequired(string username, SecurityToken token, ActiveDirectoryClient adclient, ref string otc) {
            bool passwordChangeRequired = false;
            if (token == null) {
                // Authentication might have failed due to password expiry
                if (adclient.PasswordChangeRequired(username)) {
                    //SetInfo(adclient.Error);
                    passwordChangeRequired = true;
                } else {
                    string error = adclient.Error;
                    if (!String.IsNullOrEmpty(error)) {
                        SetError(error);
                    }
                    otc = ""; // Force PINsafe login failure.
                }
            }
            return passwordChangeRequired;
        }*/

        private bool LoginSharePoint(SecurityToken token) {
            SPFederationAuthenticationModule fedmod = SPFederationAuthenticationModule.Current;
            if (fedmod == null) {
                SetError("Federation authentication module not found");
                loggedIn = false;
            } else {
                fedmod.SetPrincipalAndWriteSessionToken(token);
                loggedIn = true;
            }
            return loggedIn;
        }

        private SecurityToken SPAuthenticate(string username, string password) {
            SPContext context = SPContext.Current;
            DebugLogger logger = settings.Logger;
            if (context == null) {
                SetError("Sharepoint context not found");
                logger.WriteLine(LOG_SOURCE, "SharePoint context not found");
                return null;
            }
            String url = SharePointUrl;
            Uri currentUri = null;
            SPWebApplication webapp;
            try {
                currentUri = new Uri(url);
                webapp = SPWebApplication.Lookup(currentUri);
            } catch {
                // Ignore.
                webapp = null;
            }
            if (webapp == null) {
                if (String.IsNullOrEmpty(settings.WebApplication)) {
                    SetError("Sharepoint web application \"" + url + "\" not found");
                    logger.WriteLine(LOG_SOURCE, "Sharepoint web application \"{0}\" not found", url);
                    return null;
                }
                currentUri = new Uri(settings.WebApplication);
                webapp = SPWebApplication.Lookup(currentUri);
                if (webapp == null) {
                    SetError("Sharepoint web application not found for \"" + url + "\" or for \"" +
                             settings.WebApplication + "\"");
                    logger.WriteLine(LOG_SOURCE, "Sharepoint web application not found for \"{0}\" or for \"{1}\"", url, settings.WebApplication);
                    return null;
                }
            }
            SPIisSettings iisSettings = webapp.IisSettings[SPUrlZone.Default];
            if (iisSettings == null) {
                SetError("IIS settings not found");
                logger.WriteLine(LOG_SOURCE, "IIS settings not found");
                return null;
            }
            SPFormsAuthenticationProvider authProvider = iisSettings.FormsClaimsAuthenticationProvider;
            if (authProvider == null) {
                SetError("Authentication provider not found");
                logger.WriteLine(LOG_SOURCE, "Authentication provider not found");
                return null;
            }
            SecurityToken token = SPSecurityContext.SecurityTokenForFormsAuthentication(currentUri,
                                                                                        authProvider.MembershipProvider,
                                                                                        authProvider.RoleProvider,
                                                                                        username, password);
            return token;
        }

        private String SharePointUrl {
            get {
                StringBuilder url = new StringBuilder(Request.Url.Scheme);
                url.Append("://");
                url.Append(Request.Url.Host);
                if (!Request.Url.IsDefaultPort) {
                    url.Append(":");
                    url.Append(Request.Url.Port);
                }
                return url.ToString();
            }
        }

        private bool LoginRequired(string username, PINsafeClient client) {
            bool loginRequired = true;
            if (settings.RequiresPINsafe != null && !(bool)settings.RequiresPINsafe) {
                loginRequired = false;
            } else if (settings.AllowNonPINsafeUsers) {
                // Check if the user exists.
                if (!client.UserExists(username)) {
                    loginRequired = false;
                }
            }
            return loginRequired;
        }

        private void SetError(string message) {
            SetComment(message, CSS_ERROR_CLASS);
        }

        private void SetInfo(string message) {
            SetComment(message, CSS_INFO_CLASS);
        }

        private void SetComment(string message, string cssClass) {
            lblResult.Text = message;
            lblResult.CssClass = cssClass;
        }
    }
}
